# -*- coding: utf-8 -*-
"""
ModelSubroutines.py
Hugh Montag
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)
01/02/2024

A codebase for conducting a backward iteration endogenous grid method
of finding the optimal policy function. Household saves with one
risky asset.

This a twosector version of the model. This version attempts to incorporate
all the debugging changes I have made and serve as a template for the production
code. Any future code should be spun off from this codebase.

In this framework, households choose the nominal assets AN and
the nominal consumption CN, instead of real values

This version of the code uses a cubic spline for interpolating the policy
function during the endogenous gridpiont method. The version also uses a cubic
spline to interpolate the value function between asset gridpoints


"""

import numpy as np
import numpy.matlib
import scipy as sp
from scipy.stats import norm

import NumericalAlgo as hb1

# our primary tool for easy, efficient compiled code is Numba
from numba import jit, njit, vectorize, guvectorize

"""Part 1: Backwards iteration functions
"""


def backwards_iterate_twosec(Pi_p, uc_p, a_grid, a_grid_p, s_mat_key, 
                      i_ante=1.02, beta=0.96, bweight=0.5, eis=1):
    """Take expectations over Markov grid s and a guess for the envelope of
    value function over the grid, and back out optimal portfoio and envelope
    for today's grid
    
    State space has dimenion SxA (state by asset grid). The states are defined 
    over combinations of income and inflation rates, so I need to unpack 
    exogenous variables from each state.
    
    INFLATION IS TWOSECTOR
    
    Inputs: 
        Pi_p: (SxS array) Transition matrix from state s to state s' 
        uc_p: (SxA array) Guess for marginal utility for state s asset a combo
        a_grid: (A array) Asset grid today
        a_grid_p: (A array) Asset grid tomorrow 
        s_mat_key: (exovarXS array): States are coded on columns, exogenous
           variables on rows. 0th row is transitory income shock, 1st row is 
           permanent income shock, 2nd row is bread 
           inflation rate, 3rd row is non-bread inflation rate
        i_ante: (scalar) Fixed ex-ante nominal interest rate
        beta: (scalar) household discount factor, used for euler
        bweight (scalar): The share of the bread sector in the aggregator for
           consumption, in [0,1]        
        eis: (scalar) elasticity of intertemporal substitution, used in marg 
           utility
           
    Outputs:
        uc_new: (SxA array) Updated grid for marginal utility for state s 
            asset a combo
        a_new: (SxA array) Updated asset rule a'(s,a) for state s and asset 
            level a today
        
    """
    #Find ex-post interest rate in period t+1, for state sj. Store as a n-vector
    inflrate=(s_mat_key[2,:]**bweight)*(s_mat_key[3,:]**(1-bweight))
    i_post=i_ante/inflrate
    
    #Store ex-post interest rate on a sXa grid. Entries will be same for each a
    i_post_mat = np.matlib.repmat(i_post[:,np.newaxis],1,uc_p.shape[1])
    
    #Expected return to saving for tomorrow on s_t X a_t grid.
    #Alternative interpretation: RHS of Euler equation
    uc_nextgrid = beta * Pi_p @ (i_post_mat * uc_p)
    
    #Implied consumption this period, defined over s_t X a_t grid
    c_nextgrid = (uc_nextgrid)**(-eis)
    
    #Labor income this period (for state si)
    y = (s_mat_key[0,:] * s_mat_key[1,:])[:,np.newaxis]

    #cash on hand today, defined in terms of a_{t-1} and s_t
    coh = a_grid[np.newaxis,:] * i_post[:,np.newaxis] + y
    
    """
    #Create an empty array for the optimal savings policy a' this period
    a = np.empty(uc_p.shape)
    
    #Loop over each current state s and interpolate using cubic splines to find
    #the current asset policy
    for ss in range(s_mat_key.shape[1]):
        interpolcs = sp.interpolate.CubicSpline(c_nextgrid[ss,:] + a_grid_p, a_grid_p)
        a[ss,:] = interpolcs(coh[ss,:])
    
    
    """
    # This code conducts a linear interpolation using Matt's code
    # we have consumption c today for each a' tomorrow, so we have
    # map from coh=c+a' to a' on tomorrow's asset grid.
    # interpolate in coh to get map of coh to a' on today's grid.
    # do this separately for each s (automatic via broadcasting).
    a = hb1.interpolate_y(c_nextgrid + a_grid_p, coh, a_grid_p)
    
    
    # set constrained agents' policies to minimum asset level
    hb1.setmin(a, a_grid_p[0])
    a = np.minimum(a, a_grid_p[-1])
    
    # calculate consumption, return marginal utility and asset policy
    c = coh - a
    uc = c ** (-1/eis)
    return uc, a, c   



"""Part 2: Find optimal policy and envelope function
"""


def pol_ss_twosec(Pi, a_grid, s_mat_key, s_mat_key_true, i_ante, beta, 
                  bweight=0.5, bweighttrue=0.01,
                  eis=1, uc_seed=None, tol=1E-8, maxit=5000):
   
    """Calculate the steady state uc_p over sXa grid, given 
    the model parameters.
    
    This function iterates repeatedly using the endogenous gridpoing
    method until u'(c(s,a)), the envelope condition, converges over the grid.
    
    INFLATION IS TWO SECTOR
    
    s_mat_key: (4xN array) the key of exogenous states that agents believe going 
       forward. May be biased
    s_mat_key_true: (4xN array) the actual key of exogenous state, which agents use when
       assessing past inflation
    
    This code is shamelessly borrowed from Matt
    Pi_p, uc_p, a_grid, a_grid_p, ymean=1, i_ante=1.02, beta=0.96, eis=1
    """
    
    #Find the ratio of \frac{E^*[\pi]}{\pi} for each state S, and 
    #store in a S vector that can be broadcast
    infl_ratio_prim=s_mat_key[2,:]**bweight*s_mat_key[3,:]**(1-bweight)/(
        s_mat_key_true[2,:]**bweighttrue*s_mat_key_true[3,:]**(1-bweighttrue))
        
    infl_ratio=infl_ratio_prim[:,np.newaxis]

    s_mat_key_deflated = s_mat_key.copy()

    #Bias the real income process. Households misperceive their real cash-on-hand,
    #including their real income
    s_mat_key_deflated[0,:] = s_mat_key_deflated[0,:] / infl_ratio_prim    
    s_mat_key_deflated[1,:] = s_mat_key_deflated[1,:] / infl_ratio_prim   
    
    if uc_seed is None:
        # if no uc_seed, start by assuming 10% of coh consumed
        # beginning of period wealth depends on inflation rate, income process
        uc = (0.05*(i_ante/(s_mat_key_deflated[2,:][:,np.newaxis])*a_grid[np.newaxis,:] + 
                   (s_mat_key_deflated[0,:]*s_mat_key_deflated[1,:])[:, np.newaxis])) ** (-1/eis)          
    else:
        uc = uc_seed

    # iterate until convergence of a policy by tol, or maxit
    for it in range(maxit):
        ucnew, anew, c = backwards_iterate_twosec(Pi, uc, a_grid, a_grid, s_mat_key_deflated,
                                    i_ante, beta, bweight, eis)

#        noborrowing = (anew > a_grid[0])
        # only check convergence every 5 iterations for efficiency
        if it % 5 == 1 and hb1.within_tolerance(a, anew, tol) and hb1.within_tolerance(uc, ucnew, tol):
            break
        a = anew
        uc = ucnew
    else:
        raise ValueError(f'No convergence after {maxit} backward iterations!')

    # atrue is the actual price-deflated bond holdings the households purchase
    # in each state    
    atrue = anew * infl_ratio
    ctrue = c * infl_ratio
    
    atrue = np.minimum(atrue, a_grid[-1])
    
    return ucnew, atrue, ctrue


def test_Euleruc(uc, apol, Pi_p, a_grid, s_mat_key, i_ante, beta, bweighttrue):
    """
    Given the marginal utilites for (S,A) and an optimal policy function apol,
    test whether the Euler equation holds.
    
    Two sector formulation, with permanent and transitory income
    """
    
    
    #Copy over the marginal consumptions
    euLHS = uc.copy()
    ucnext = uc.copy()
    
    #Find ex-post interest rate in period t+1, for state sj. Store as a 2d array
    inflrate=(s_mat_key[2,:]**bweighttrue)*(s_mat_key[3,:]**(1-bweighttrue))
    i_post=i_ante/inflrate    
    i_post_mat = np.matlib.repmat(i_post[:,np.newaxis],1,uc.shape[1])
    
    #Interpolate the policy function
    aqi, aqpi = hb1.interpolate_coord(a_grid, apol)

    euRHS = np.zeros(ucnext.shape)    
    for ss in range(ucnext.shape[0]):
        for aa in range(ucnext.shape[1]):
            dd = (i_post_mat[:,aa] * (aqpi[ss,aa] * 
                            ucnext[:, aqi[ss,aa]] + (1-aqpi[ss,aa]) * ucnext[:, aqi[ss,aa]+1]))[:, np.newaxis]
            dd1 = beta * Pi_p[ss,:][np.newaxis, :] 
             
            euRHS[ss,aa] = dd1 @ dd

    return euRHS, euLHS

"""Part 3: Find endogenous distribution in equilibrium
   These functions work without requiring any details about how many assets,
   or the weight on bread.
"""

@njit   
def forward_iterate(D, Pi_T, a_pol_i, a_pol_pi):
    """Update distribution of agents over state space S*A by
    iterating forward asset policy, then state.
    
    Asset policy is represented using interpolated coordinates
    a_pol_i and a_pol_pi: when an agent tries to choose a level
    of assets in between two gridpoints, we update the
    distribution as if she chose a lottery between those two
    gridpoints that gives the same level of assets on average.
    
    Parameters
    ----------
    D : array (S*A), beginning-of-period distribution s_t, a_(t-1)
    Pi_T : array (S*S), transpose Markov matrix for s_t to s_(t+1)
    a_pol_i : int array (S*A), left gridpoint of asset policy
    a_pol_pi : array (S*A), share left gridpoint in asset policy
    
    Returns
    ----------
    Dnew : array (S*A), beginning-of-next-period dist s_(t+1), a_t
    """
    
    # first create Dnew from updating asset state
    Dnew = np.zeros_like(D)
    for s in range(D.shape[0]):
        for i in range(D.shape[1]):
            apol = a_pol_i[s,i]
            api = a_pol_pi[s,i]
            d = D[s,i]
            Dnew[s, apol] += d*api
            Dnew[s, apol+1] += d*(1-api)
    
    # then use transpose Markov matrix to update 's'
    Dnew = Pi_T @ Dnew
    
    return Dnew

def dist_ss(Pi, a_grid, a_pol,
            D_seed=None, pi_seed=None, tol=1E-10, maxit=10**5):
    """Iterate to find steady-state distribution of s_t, a_(t-1).
    
    Parameters
    ----------
    Pi : array (S*S), steady-state Markov matrix
    a_grid : array (A), steady-state asset grid
    a_pol : array (S*A), steady-state asset policy function
    D_seed : [optional] array (S*A), seed for s*a distribution
    pi_seed : [optional] array (S), seed for s distribution
    tol : [optional] float, stop if iterations change less
           than tol in Linfty norm
    maxit : [optional] int, stop if more than maxit iterations
    
    Returns
    ----------
    D : array (S*A), steady-state s*a distribution of agents
    """
    
    if D_seed is None:
        # compute separately stationary dist of s, to start there
        # assume a uniform distribution on assets otherwise
        pi = hb1.stationary(Pi, pi_seed)
        D = np.tile(pi[:, np.newaxis],
                    (1, a_grid.shape[0])) / a_grid.shape[0]
    else:
        D = D_seed
    
    # obtain interpolated-coordinate asset policy rule
    a_pol_i, a_pol_pi = hb1.interpolate_coord(a_grid, a_pol)
    
    # to make matrix multiplication more efficient, make
    # separate copy of Pi transpose so memory right alignment
    Pi_T = Pi.T.copy()
    
    # iterate until convergence by tol, or maxit
    for it in range(maxit):
        Dnew = forward_iterate(D, Pi_T, a_pol_i, a_pol_pi)
        
        # only check convergence every 5 iterations for efficiency
        if it % 5 == 0 and hb1.within_tolerance(D, Dnew, tol):
            break
        D = Dnew
    else:
        Dnew = forward_iterate(D, Pi_T, a_pol_i, a_pol_pi)
        display('Mean-squared difference is: {0:.15f}'.format(np.sum((D-Dnew)**2)))
        display('Maximum difference is: {0:.12f}'.format(np.max(np.abs(D-Dnew))))
        display('Magnitude of maximum difference is: {0:.12f}'.format(np.log10(np.max(np.abs(D-Dnew)))))        
        raise ValueError(f'No convergence after {maxit} forward iterations!')
    D = Dnew
    
    return D
    
"""Part 4: Value function calculation
   Now that I have the policy function for my problem, calculate the value function
"""

@njit
def backward_value_twosec(Vnext, Pi, agrid, a_pol_i, a_pol_pi, s_mat_key, i_ante, beta, bweight, eis=1):
    """Update value function over state space S*A by iterating value function
    backwards using the optimal asset policy
    
    In order to update the value function, we need to calculate the 
    1) within-period utility due to consumption
    2) the expected discounted value function for the next period
    V=u(C)+\beta E[V]
    Therefore, this program calls on all the parameters used in the 
    consumer utility function.
    
    Asset policy is represented using interpolated coordinates
    a_pol_i and a_pol_pi: when an agent tries to choose a level
    of assets in between two gridpoints, we update the
    distribution as if she choses a lottery between those two
    gridpoints that gives the same level of assets on average.
    
    INFLATION IS A COMBINATION OF TWO SECTORS
    
    Parameters:
    -----------
    Vnext: array (S*A), value function to be iterated on. This will represent
    the value function for next period
    Pi: array (S*S), Markov matrix between combined state spaces between s_t 
    and s_{t+1}
    agrid: array (A), array of asset gridpoints
    a_pol_i: int array (S*A), left gridpoint index for asset policy
    s_mat_key: array (numS*S) Stores values for state vars at each state. 
    s_mat_key[0,:] is permanent labor income, 1 is transitory income, 
        2 is sector 1 inflation, 3 is sector 2 inflation
    i_ante: float, the nominal return on bonds
    beta: float, the discount rate for consumer utility
    bweight: float, the aggregation weight for inflation (sector 1 vs sector 2) 
    eis: float, the elasticity of intertemporal substitution for consumers
    
    Returns:
    -----------
    Vnew: array (SxA), new value function
    """
        
    Vnew=np.zeros_like(Vnext)
    for ss in range(Vnew.shape[0]):
        for aa in range(Vnew.shape[1]):
            apol=a_pol_i[ss,aa]
            apolpi=a_pol_pi[ss,aa]
            anext=agrid[apol]*apolpi+agrid[apol+1]*(1-apolpi)
            
            Vtemp=apolpi*Vnext[:,apol]+(1-apolpi)*Vnext[:,apol+1] #Future utility
            #Wealth at start of period
            dd1 = s_mat_key[2,ss]**bweight
            dd2 = s_mat_key[3,ss]**(1-bweight)            
            startwealth=agrid[aa]*i_ante/(dd1 * dd2)
            #startwealth=agrid[aa]*i_ante/(s_mat_key[2,ss]**bweight*s_mat_key[3,ss]**(1-bweight))
            #Intra-period utility. (1+i)a+y-a'=c
            y = s_mat_key[0,ss] * s_mat_key[1,ss]
            if (startwealth+y-anext) < 0:
                print(ss,aa)
                print(s_mat_key[2,ss])
                print(startwealth)
                print(bweight)

                print(dd1)
                print(dd2)
                print(type(dd1 * dd2))
                print(dd1 * dd2)
                print(startwealth + y - anext)
                raise Exception("Negative Consumption")
            
            if eis != 1:
                intrautil=(startwealth+y-anext)**(1-1/eis) / (1-1/eis)
            else:
                intrautil=np.log(startwealth+y-anext)
                
            #Discount and add within-period utility
            Vnew[ss,aa]=intrautil + beta * Pi[ss,:] @ Vtemp 
            
    return Vnew
    
def backward_value_cubic(Vnext, Pi, agrid, a_pol, s_mat_key, i_ante, beta, bweight, eis=1):
    """Update value function over state space S*A by iterating value function
    backwards using the optimal asset policy
    
    In order to update the value function, we need to calculate the 
    1) within-period utility due to consumption
    2) the expected discounted value function for the next period
    V=u(C)+\beta E[V]
    Therefore, this program calls on all the parameters used in the 
    consumer utility function.
    
    Asset policy is represented using a cubic spline to interpolate 
    between the gridpoints on the asset grid. Cubic interpolation slows down this
    process a lot, so this function shouldn't be called in a loop
    
    INFLATION IS A COMBINATION OF TWO SECTORS
    
    Parameters:
    -----------
    Vnext: array (S*A), value function to be iterated on. This will represent
    the value function for next period
    Pi: array (S*S), Markov matrix between combined state spaces between s_t 
    and s_{t+1}
    agrid: array (A), array of asset gridpoints
    a_pol_i: int array (S*A), left gridpoint index for asset policy
    s_mat_key: array (numS*S) Stores values for state vars at each state. 
    s_mat_key[0,:] is permanent labor income, 1 is transitory income, 
        2 is sector 1 inflation, 3 is sector 2 inflation
    i_ante: float, the nominal return on bonds
    beta: float, the discount rate for consumer utility
    bweight: float, the aggregation weight for inflation (sector 1 vs sector 2) 
    eis: float, the elasticity of intertemporal substitution for consumers
    
    Returns:
    -----------
    Vnew: array (SxA), new value function
    """
    
    Vnew=np.zeros_like(Vnext)
    Vinterpolate_list = []
    for ss in range(Vnew.shape[0]):
        if (np.diff(Vnext[ss,:]) <= 0).any():
            print('Value function is not monotonic increasing in asset grid')
            
        Vinterpolate_list.append(sp.interpolate.CubicSpline(agrid, Vnext[ss,:]))    
    
    for ss in range(Vnew.shape[0]):
        for aa in range(Vnew.shape[1]):
            anext = a_pol[ss,aa]
            #Create empty array for V(S',A')
            Vtemp = np.empty(Vnext.shape[0])
            #Interpolate using a cubic spline for each S'
            for sspr in range(Vnew.shape[0]):
                Vtemp[sspr] = Vinterpolate_list[sspr](a_pol[ss,aa])
            
            #Wealth at start of period
            startwealth=agrid[aa]*i_ante/(s_mat_key[2,ss]**bweight*s_mat_key[3,ss]**(1-bweight))
            #Intra-period utility. (1+i)a+y-a'=c
            y = s_mat_key[0,ss] * s_mat_key[1,ss]
            if eis != 1:
                intrautil=(startwealth+y-anext)**(1-1/eis) / (1-1/eis)
            else:
                intrautil=np.log(startwealth+y-anext)
                
            #Discount and add within-period utility
            Vnew[ss,aa]=intrautil + beta * Pi[ss,:] @ Vtemp 
            
    return Vnew    
    
        
def val_ss_twosec(Pi, a_grid, a_pol, s_mat_key, i_ante, 
                  beta, bweight, eis=1, V_seed=None, tol=1E-10, maxit=10**4):
    """Iterate backwards to find value function over state-asset grid s*A
    
    This function performs backward value iteration repeatedly until the
    value functions convege. 
    
        INFLATION IS A COMBINATION OF TWO SECTORS
    
    Parameters:
    -----------
    Pi: array (S*S), Markov matrix between combined state spaces between s_t 
    and s_{t+1}
    a_grid: array (A), array of asset gridpoints
    a_pol_i: int array (S*A), left gridpoint index for asset policy
    s_mat_key: array (numS*S) Stores values for state vars at each state. 
    s_mat_key[0,:] is permanent labor income, 1 is transotiry labor income,
        2 is sector 1 inflation, 3 is sector 2 inflation
    i_ante: float, the nominal return on bonds
    beta: float, the discount rate for consumer utility
    bweight: float, the aggregation weight for inflation (emphasis on sector 1)
    eis: float, the elasticity of intertemporal substitution for consumers
    V_seed: array (S*A), initial value function to start backwards iteration
    tol: float, tolerance to check for convergence
    maxit: int, maximum number of iterations 
    Returns:
    -----------
    Vout: array (SxA), new value function
    """
    
    if V_seed is None and eis != 1:
        # if no V_seed, assume that value function is value of consuming
        # entire remaining wealth
        V = ((i_ante/(s_mat_key[2,:]**bweight*s_mat_key[3,:]**(1-bweight))[:,np.newaxis]*
            a_grid[np.newaxis,:]+((s_mat_key[0,:]*s_mat_key[1,:])[:, np.newaxis])) ** (1-1/eis) / 
            (1-1/eis))
    elif V_seed is None and eis == 1:
        V = np.log(i_ante/(s_mat_key[2,:]**bweight*s_mat_key[3,:]**(1-bweight))[:,np.newaxis]*
            a_grid[np.newaxis,:]+((s_mat_key[0,:]*s_mat_key[1,:])[:, np.newaxis]))
    else:
        V = V_seed
        
    # obtain interpolated-coordinate asset policy rule
    a_pol_i, a_pol_pi = hb1.interpolate_coord(a_grid, a_pol)
    
    print("Start Value Function Iteration")
    # iterate until convergence by tol, or maxit
    for it in range(maxit):
        Vnew = backward_value_twosec(V, Pi, a_grid, a_pol_i, a_pol_pi, s_mat_key, 
                i_ante, beta, bweight, eis)
            
        # only check convergence every 5 iterations for efficiency
        if it % 5 == 0 and hb1.within_tolerance(V, Vnew, tol):
            print("# of iterations for value function is {0:4d}".format(it))
            break
        V = Vnew
    else:
        raise ValueError(f'No convergence after {maxit} forward iterations!')
    
    #Vinterpolated = backward_value_cubic(Vnew, Pi, a_grid, a_pol, s_mat_key, 
    #            i_ante, beta, bweight, eis)
        
    #V=Vinterpolated
    V=Vnew
    
    return V
    

"""Part 5: Callibration calculation
   With the policy function and value function code above, write a wrapper 
   for both and alter nominal interest rates to match targets
"""

           
def household_ss_twosec(Pi_p, agrid, pr_mat_key_true, i_ante, beta, bweight_true, ss_index, 
                 eis=1, pr_mat_key_bias=None, bweight_bias=None,
                 uc_seed = None, D_seed = None, pi_seed = None):
    """Wrapper to backward iteration pol_ss to get steady-state pol,
    dist_ss to get steady-state distribution, reports idiosyncratic
    and aggregate results.
    """
    #display(i_ante)
    if pr_mat_key_bias is None:
        pr_mat_key_bias=pr_mat_key_true
        
    if bweight_bias is None:
        bweight_bias=bweight_true

    # backward iterate till convergence
    uc_bias,a_bias,c_bias=pol_ss_twosec(Pi_p, agrid, pr_mat_key_bias, pr_mat_key_true,
        i_ante, beta, bweight_bias, bweight_true, 
        eis, uc_seed=None, tol=1E-10, maxit=10000)    

    # forward iterate with biased policy until convergence
    D=dist_ss(Pi_p, agrid, a_bias)   
 
    #Find the total assets and consumption in steady state
    #Steady state distribution if biased asset policies
    SSdist=D[ss_index,:]/np.sum(D[ss_index,:])
        
    # return handy dict with results and inputs
    inputs = {'Pi_p': Pi_p, 'agrid': agrid, 'pr_mat_key_true': pr_mat_key_true,
              'bweight_true': bweight_true, 'pr_mat_key_bias': pr_mat_key_bias,
              'bweight_bias': bweight_bias,
              'i_ante': i_ante, 'eis': eis, 'beta': beta}
              
    results = {'D': D, 'uc_bias': uc_bias, 'a_bias': a_bias, 
               'c_bias': c_bias,
               'Atot': np.sum(SSdist @ agrid)}
               
    #display(np.sum(SSdist @ agrid))
    display('Eval Iteration Run, beta = {:07.6f}'.format(beta))
    return {**inputs, **results}
    
    
    
#===============================    
def calibration_simple(A_target, Pi_p, agrid, pr_mat_key_true, i_ante, bweight_true, 
                 xmean, eis=1, beta_range=(0.8,0.999), beta_tol=1E-8,
                 pr_mat_key_bias=None, bweight_bias=None,
                 uc_seed = None, D_seed = None, pi_seed = None):
    """Callibrate the nominal interest rate to match a level bias
    """
    
    if bweight_bias is None:
        bweight_bias=bweight_true
    
    beta_low = beta_range[0]    
    beta_high = beta_range[1] * i_ante/(xmean[2]**bweight_true*xmean[3]**(1-bweight_true))    

    ss_index=[ii for ii in range(pr_mat_key_true.shape[1]) if np.equal(pr_mat_key_true[2:,ii],xmean[2:]).all()]
    
    def evalhh(beta):
        return household_ss_twosec(Pi_p, agrid, pr_mat_key_true, i_ante, beta,
                            bweight_true, ss_index, eis, pr_mat_key_bias,
                            bweight_bias, uc_seed, D_seed, pi_seed)
    
    beta, solver_data = sp.optimize.brentq(
        lambda beta: evalhh(beta)['Atot'] - A_target,
        beta_low, beta_high, full_output = True,
        xtol=beta_tol)
            
    # wasteful extra evaluation at end to get all data
    # can avoid, but with some more effort
    results = evalhh(beta)
    display('Calibrated beta = {:07.6f}'.format(results['beta']))
    display('Atot = {:06.4f}'.format(results['Atot']))
            
    # Find value function
    #Find the value function for biased policy in the true world
    val_bias = val_ss_twosec(Pi_p, agrid, results['a_bias'], pr_mat_key_true, i_ante, 
        beta, bweight_true, eis)

    #Save extra data to results
    results['solver_data'] = solver_data
    results['val_bias'] = val_bias
    return results