# -*- coding: utf-8 -*-
"""
NumericalAlgo.py
Hugh Montag
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)
01/02/2024

Some code is used with permission from Prof. Rognlie

Functions in this codebase:
agrid
doublej
variance
stationary
within_tolerance
secant_method
markov_tauchen
markov_tauchen_multivariate
markov_tauchen_mean
tauchen_untrans
interpolate_y
interpolate_matt
interpolate_coord
setmins
"""

import numpy as np
import numpy.matlib
import scipy as sp
from scipy.stats import norm

# our primary tool for easy, efficient compiled code is Numba
from numba import jit, njit, vectorize, guvectorize

"""Part 1: defining exogenous grids and states.

Nothing here very performance-sensitive, so we strive
for clarity."""

def agrid(amax,N,amin=0,pivot=0.25):
    """Grid with a+pivot evenly log-spaced between
    amin+pivot and amax+pivot.
    """
    a = np.geomspace(amin+pivot,amax+pivot,N) - pivot
    a[0] = amin # make sure *exactly* equal to amin
    return a


def doublej(a1,b1):
    """Computes an infinite sum of matrices given by
    
    V = \sum_j (a1^j) b1 (a1^j)'
    where a1_j and b1_j are (nxn) matrices whose eigenvalues are
    within the unit circle.
    
    This code is copied from linquist-sargent, and updated for Python.
    They use a "doubling algorithm" to cut down on computation time. 
    I will forgo that.
    
    Let a1(j)=a1(j-1)*a1(j-1), where a1(0)=a1, the input
    Let V(j)=V(j-1)+a1(j-1)V(j-1)a1(j-1)', where V(0)=v1, the input
    """

    alpha0=a1
    gamma0=b1
    diff=10
    numtrial=1
    
    eigval,eigvec=np.linalg.eig(alpha0)
    if np.max(np.abs(eigval))>1:
        print('Error Error!')
        print('At least one eigenvalue lies outside the unit circle')
        return
    
    while diff>10**-10:
        alpha1=alpha0 @ alpha0
        gamma1=gamma0 + alpha0 @ gamma0 @ alpha0.transpose()
        
        diff=np.max(gamma1-gamma0)
        gamma0=gamma1
        alpha0=alpha1

        if numtrial>10**10:
            print('DoubleJ solution to Discrete Lyapunov equation not converging')
            return
            
        numtrial+=numtrial
  
    return gamma1, numtrial            

def variance(x, pi):
    """Variance of x distributed with density pi"""
    return np.sum(pi*(x-np.sum(pi*x))**2)

def stationary(Pi, pi_seed=None, tol=1E-11, maxit=10_000):
    """Iterate to find invariant distribution of Markov chain.00
    
    Parameters
    ----------
    Pi : array (S*S), Markov chain
    pi_seed : [optional] array (S), seed for iteration
    tol : [optional] float, stop if iterations change less
           than tol in Linfty norm
    maxit : [optional] int, stop if more than maxit iterations
    
    Returns
    ----------
    pi : array (S), invariant distribution of Markov chain
    """
    if pi_seed is None:
        pi = np.ones(Pi.shape[0])/Pi.shape[0]
    else:
        pi = pi_seed
        
    for it in range(maxit):
        pi_new = pi @ Pi
        if np.max(np.abs(pi_new-pi)) < tol:
            break
        pi = pi_new 
    pi = pi_new
    
    return pi

@njit
def within_tolerance(x1, x2, tol):
    """Efficiently test max(abs(x1-x2)) <= tol for same-dim x1, x2"""
    # implement by obtaining flattened views using ravel, then looping
    y1 = x1.ravel()
    y2 = x2.ravel()
    
    for i in range(y1.shape[0]):
        if np.abs(y1[i] - y2[i]) > tol:
            return False
    return True

def secant_method(myfunct,x0,x1,maxtrial=10**5,conv_tol=10**-5):
    """Apply the secant method to root-find myfunct. 
    
    myfunct is a scalar function, where any parameters or data
    has already been loaded in. Note that I am assuming monotonicity here
    
    Use initial points x0 and x1 to start the secant method. 
    I assume that the domain is [0,1], so this will check for boundary
    conditions.
    """
    
    """
    #Check for boundary conditions
    if min(myfunct(0),myfunct(1))>0:
        print("Function is positive over [0,1]. No zeros")
        print("Returning best boundary solution")
        if myfunct(0)<myfunct(1):
            return 0
        else:
            return 1
    elif max(myfunct(0),myfunct(1))<0:
        print("Function is negative over [0,1]. No zeros")
        print("Returning best boundary solution")
        if myfunct(0)<myfunct(1):
            return 1
        else:
            return 0    
    """
    #Begin looping over secant method
    numtrial=1
    diff=10
    while numtrial<maxtrial and diff>conv_tol:
        #Update guess\
        #try:
        
        x2=x1-(x1-x0)/(myfunct(x1)-myfunct(x0))*myfunct(x1)
        
        #except:
        #    print('Secant method has converged to a non-zero point for some reason')
        #Bound x2 by [0,1]. We know a 0 exists, so this is just overshooting
        """
        if x2<0:
            x2=0
        elif x2>1:
            x2=1
        """
        #See how close myfunc(x2) gets to 0
        diff=np.abs(myfunct(x2)-0)
        
        #Store previous guesses and begin next loop
        x0=x1
        x1=x2    
        numtrial+=1
 
    #Return either an error message or the results 
    if numtrial>=maxtrial:
        print('Secant method did not converge')
        print('# of loops exceeded maximum number of loops')
        print(x1, x0, numtrial)
    else: 
        return x1
        

def markov_rouwenhorst(rho, sigma, N=7):
    """Rouwenhorst method analog to markov_tauchen"""
    
    # parametrize Rouwenhorst for n=2
    p = (1 + rho) / 2
    Pi = np.array([[p, 1 - p], [1 - p, p]])

    # implement recursion to build from n=3 to n=N
    for n in range(3, N + 1):
        P1, P2, P3, P4 = (np.zeros((n, n)) for _ in range(4))
        P1[:-1, :-1] = p * Pi
        P2[:-1, 1:] = (1 - p) * Pi
        P3[1:, :-1] = (1 - p) * Pi
        P4[1:, 1:] = p * Pi
        Pi = P1 + P2 + P3 + P4
        Pi[1:-1] /= 2
        
    # invariant distribution and scaling
    pi = stationary(Pi)
    s = np.linspace(-1, 1, N)
    s *= (sigma / np.sqrt(variance(s, pi)))
    y = np.exp(s)/np.sum(pi*np.exp(s))
    
    return s, pi, Pi, y


def markov_tauchen(rho, sigma, N=7, m=3):
    """Tauchen method discretizing AR(1) s_t = rho*s_(t-1) + eps_t.
        
    Parameters
    ----------
    rho : float, persistence
    sigma : float, unconditional sd of s_t
    N : int, number of states in discretized Markov process
    m : float, discretized s goes from approx -m*sigma to m*sigma
    
    Returns
    ----------
    s : array (N), states in discretized process
    pi : array (N), stationary distribution of discretized process
    Pi : array (N*N), Markov matrix for discretized process
    y : array (N), states proportional to exp(s) s.t. E[y] = 1
    """
    
    # make normalized grid, start with cross-sectional sd of 1
    s = np.linspace(-m, m, N)
    ds = s[1] - s[0]
    sd_innov = np.sqrt(1-rho**2)
    
    # standard Tauchen method to generate Pi given N and m
    Pi = np.empty((N, N))
    Pi[:, 0] = norm.cdf(s[0] - rho*s + ds/2, scale=sd_innov)
    Pi[:, -1] = 1 - norm.cdf(s[-1] - rho*s - ds/2, scale=sd_innov)
    for j in range(1, N - 1):
        Pi[:, j] = (norm.cdf(s[j] - rho*s + ds/2, scale=sd_innov) - 
                    norm.cdf(s[j] - rho*s - ds/2, scale=sd_innov))
        
    # invariant distribution and scaling
    pi = stationary(Pi)
    #display(sigma / np.sqrt(variance(s, pi)))
    s *= (sigma / np.sqrt(variance(s, pi)))
    y = np.exp(s)/np.sum(pi*np.exp(s))
    
    return s, pi, Pi, y


def markov_tauchen_multivariate(A, Sigma, Narray, xmean=np.array([0]), m=3):
    """Multi-variate Tauchen-method discretizing VAR(1) 
    s_t=A s_{t-1}+\epsilon_{t}
    
    Uses discretization method of 
    http://www.fperri.net/TEACHING/macrotheory08/numerical.pdf
    
    Specifically, decompose the variance-covariance matrix of \epsilon_t 
    into \Sigma=Q \Lambda Q', and premultiply VAR by Q to get a version of the VAR with
    orthogoal shocks. Then discretize this transformed VAR using Tauchen's AR(1) method 
    
    Let n = # of variables, N(i) = # of grid points in ith variable.
    Let Prod(N) = the number of grid points of the combined variable space,
       i.e. \Pi_i N(i). 3 variables with 4 gridpoints each has Prod(N)=64
           
    Parameters:
        A (array nxn): VAR coefficient. In practice a diagonal array of persistences
        Sigma (array nxn): The variance-covariance matrix of the shocks
        Narray (array n): An array for the number of gridpoints for each transformed variable. 
           Note that this is NOT the number of untransformed gridpoints 
        ybar (array n): Mean of each of the exogenous variables. If no entry, defaults
           to 0's.
        m (integer): The discretized range for each variable, 
            from -m*sigma to m*sigma
            
    Returns:
        Pr_mat (array Prod(N)xProd(N)): Transition matrix between discretized
            Markov Chain
        Pr_mat_key (array nxProd(N)): A key such that the entry (:,j) returns an array
            contains the states for all the variables
        zbar (array nXN): A matrix s.t. zbar(i,1:N(i)) contains the grid for 
            variable i
    """
    n=A.shape[0]   #The number of state variables to track
    
    #Find the number of grid points for all variables
    prodn = 1
    for ii in range(np.size(Narray)):
        prodn *= Narray[ii]

    #Make a matrix to contain the grids for each point
    
    #Transform the VAR process from y to orthogonal \tilde{y}
    Lambda,Q=sp.linalg.eig(Sigma)   #Find eigenvalue decomposition of variance-covariance
    #Note that Q is the matrix of column eigenvectors
    if np.isreal(Lambda).all()==False:
        print('Error Error')
        print('Variance-covariance matrix of shocks is not positive-definite')
        return
    else:
        Lambda=Lambda.real
    
    Atilde=Q.transpose() @ A @ Q    #Transform the markov process coefficients by Q' A Q
    Ltilde=np.diag(Lambda)          #Transform the markov process variance-covariance matrix

    #Note: \tilde{y} = Q' @ y, y = Q @ \tilde{y}

    #Find the unconditional variance-covariance matrix for transformed VAR \tilde{y}
    varYtilde,_=doublej(Atilde,Ltilde)  

    sdYtilde=np.sqrt(np.diag(varYtilde))   #Unconditional std of transformed variables    

    #Grid that will contains values for variables    
    #Each row corresponds to a variable, columns correspond to possible values
    zbar_tilde=np.zeros([n,np.max(Narray)])    
    dsstep=np.zeros(n)   #Size of steps between gridpoints for each variable
    
    Pr_mat_key_tilde=np.empty([n,prodn])    #Make a key for the transformed variables (actual values)    
    Pr_mat_pos_tilde=np.empty([n,prodn])    #Make a key for the transformed variables (grid index 0:N)
    

    #Construct a grid for the discretized value for each variable,
    #then kronecker them together to get one combine grid  
    cumprod = 1
    for ii in range(n):
        cumprod *= Narray[ii]
        
        # make a transformed grid for each of the n variables 
        zbar_tilde[ii,:Narray[ii]]=np.linspace(-m*sdYtilde[ii],m*sdYtilde[ii],Narray[ii])
        dsstep[ii]=zbar_tilde[ii,1]-zbar_tilde[ii,0]
        
        #Temp var, # of 1's to make the kronecker come out correctly
        onesii=np.ones([1,prodn // cumprod])    
        #Fill the key with the values of the variables. Rows are variable, columns are states
        Pr_mat_key_tilde[ii,:] = np.matlib.repmat(np.kron(zbar_tilde[ii,:Narray[ii]],onesii),
                                    1,cumprod // Narray[ii])        
        #Fill the index of the variables on the grid. Rows are variables, columns are states
        Pr_mat_pos_tilde[ii,:]=np.matlib.repmat(
                np.kron(np.arange(Narray[ii],dtype=np.int16),onesii),1, cumprod // Narray[ii])    
    
    
    Pr_mat_pos_tilde=Pr_mat_pos_tilde.astype(int)   #Indices should be int, not float
    """
    Discretized Transition matrix Pi:
    The probability that variable k takes it's jth value next period given that it
    currently has it's ith value depends on the other transformed variables as well. 
    Therefore, we cannot evaluate the probability matrices of the transformed variables
    independently. Tauchen notes this when he says that \mu_k depends on the whole VAR.
    
    Define Pi over the combined transformed origin and destination states, s.t. it is 
    prodnXprodn.
    
    I will calculate the probability of transforming from each origin combined state
    to each destination combined state.
    """
    #Create transition matrix for the discretized transformed VAR
    #The matrix will be defined over the tensor of states, prodn x prodn
    Pi=np.ones([prodn,prodn])
    #Let ii denote the origin state (across all variables), and kk the destination state
    for ii in range(prodn):
        for kk in range(prodn):
            
            origkey=Pr_mat_key_tilde[:,ii]   #Variable values at origin combined state
            destpos=Pr_mat_pos_tilde[:,kk]   #Variable indicies at the destination combined state.
            #We need indices so we can evaluate if we are at a boundary 
            #(i.e. index=0 or -1 for a variable)

            #Note that Atilde @ origkey should give an array of mean values of 
            #each variable, equivalent to rho*y_{t-1} in AR(1)            
            ymean = Atilde @ origkey
            for jj in range(n):
                sd_innov=np.sqrt(Ltilde[jj,jj])
                zbarjj=zbar_tilde[jj, :Narray[jj]]   #Variable jj's values in different states
                #print(ii,jj,kk)

                
                if destpos[jj]==0:    #If variable jj is at the lowest value in destination state kk
                    Pi[ii,kk] *= norm.cdf(zbarjj[0] - ymean[jj] + dsstep[jj]/2, 
                        scale=sd_innov) 
                elif destpos[jj]==(Narray[jj]-1):
                    Pi[ii,kk] *= 1 - norm.cdf(zbarjj[-1] - ymean[jj] - dsstep[jj]/2, 
                        scale=sd_innov)                   
                else:
                    Pi[ii,kk] *= (norm.cdf(zbarjj[destpos[jj]] - ymean[jj] + 
                        dsstep[jj]/2, scale=sd_innov) - norm.cdf(zbarjj[destpos[jj]] - 
                        ymean[jj] - dsstep[jj]/2, scale=sd_innov))
                     

    #Find the stationary distribution of combined states
    pi_stationary=stationary(Pi)

    #Multiply the Pr_mat_key_tilde to get the standard deviation of the unconditional
    #transformed variables to match sdYtilde
    #Only use this if I am trying to match the unconditional standard variation 
    #for ii in range(n):
    #    Pr_mat_key_tilde[ii,:] *= (sdYtilde[ii] / np.sqrt(variance(Pr_mat_key_tilde[ii,:], pi_stationary))) 

    #Untransform the grid Pr_mat_key_tilde, so that I have a nXN grid of variables and
    #combined states  
    Pr_mat_key = Q @ Pr_mat_key_tilde
    
    #Add the means to Pr_mat_key
    if np.array_equal(xmean,np.array([0])):
        xmean1=np.zeros([Pr_mat_key.shape[0],1])
    else:
        xmean1=xmean[:,np.newaxis]
        
    Pr_mat_key=Pr_mat_key + xmean1
    

    
    #return zbar_tilde, Pr_mat_key_tilde, Pr_mat_pos_tilde, Pi, pi_stationary 
    return zbar_tilde, Pr_mat_key, Pr_mat_key_tilde, Pi, pi_stationary, Q


def markov_tauchen_mean(Pr_mat_key, xmean, oldmean=None):
    """Change the mean of a Markov process, without changing it's transition
    probabilities.
    
    numexo is the number of exogenous state variables. S is the combined 
    exogenous state space.
    
    Parameters
    ----------
    Pr_mat_key : array (numexo X S), values of exogenous state values
    xmean : array (numexo), new mean of exogenous state variables.
    oldmean : array (numexo), old mean of exogenous state variables
    
    Returns 
    ----------
    Pr_new_key : array (numexo X S), values of exogenous state values
       with new mean
    """
    
    #Find the change in the means 
    if oldmean is None:
        oldmean = np.mean(Pr_mat_key, axis=1)
        
    meandiff = (xmean-oldmean)[:, np.newaxis]
    
    Pr_new_key = Pr_mat_key + meandiff
    
    return Pr_new_key
    

def tauchen_untrans(statevec,Q):
    """Given a transformed state vector "statevec", untransform it to values for the original
    state variables (i.e. y, inflation).
    
    statevec should be a 1-dim array, or a column vector
    """
    return Q.transpose() @ statevec


"""Part 1A: Preliminary code for backward iteration endogenous gridpoint method

Backwards iteration is computationally intensive and will be called many 
times. The code below should be efficient.

As before, this code is borrowed from Prof Rognlie
"""

# Numba's guvectorize decorator compiles and allows function to be
# automatically broadcast by NumPy when dimensions differ
@guvectorize(['void(float64[:], float64[:], float64[:], float64[:])'], 
             '(n),(nq),(n)->(nq)')
def interpolate_y(x, xq, y, yq):
    """Efficient linear interpolation exploiting monotonicity.
    
    Complexity O(n+nq), so most efficient when x and xq have
    comparable number of points. Extrapolates linearly when
    xq out of domain of x.
    
    Parameters
    ----------
    x : array (n) ascending data points
    xq : array (nq) ascending query points
    y : array (n) data points
    yq : array (nq) empty to be filled with interpolated points
    """
    nxq, nx = xq.shape[0], x.shape[0]
    
    xi = 0
    x_low = x[0]
    x_high = x[1]
    for xqi_cur in range(nxq):
        xq_cur = xq[xqi_cur]
        while xi < nx-2:
            if x_high >= xq_cur:
                break
            xi += 1
            x_low = x_high
            x_high = x[xi+1]

        xqpi_cur = (x_high-xq_cur) / (x_high-x_low)
        yq[xqi_cur] = xqpi_cur*y[xi] + (1-xqpi_cur)*y[xi+1]

@njit
def interpolate_matt(x, xq, y):
    """We have values at x and y, where x is in increasing order, get values yq for xq,
    where we aren't making any monotonicity assumptions on xq"""

    n = len(x)
    nq = len(xq)
    yq = np.empty(nq)
    for iq in range(nq):
        if xq[iq] < x[0]:
            ilow = 0
        elif xq[iq] > x[-2]:
            ilow = n-2
        else:
            # start binary search
            # should end with ilow and ihigh exactly 1 apart, bracketing variable
            ihigh = n-1
            ilow = 0
            while ihigh - ilow > 1:
                imid = (ihigh + ilow) // 2
                if xq[iq] > x[imid]:
                    ilow = imid
                else:
                    ihigh = imid
        
        xqpi = (x[ilow+1] - xq[iq]) / (x[ilow+1] - x[ilow])
        yq[iq] = xqpi * y[ilow] + (1 - xqpi) * y[ilow + 1]

    return yq

#Interpolation function for forward iteration
# for speed and memory savings, we assign coordinates to uint32,
# an unsigned 32-bit integer (max ~4 billion)
@guvectorize(['void(float64[:], float64[:], uint32[:], float64[:])'],
             '(n),(nq)->(nq),(nq)')
def interpolate_coord(x, xq, xqi, xqpi):
    """Efficient linear interpolation exploiting monotonicity like
    interpolate_rule and with nearly-identical code, but rather than
    giving values of y at query points, gives interpolated
    "coordinates" xqi and xqpi:
    
    xq = xqpi*x[xqi] + (1-xqpi)*x[xqi+1]
    
    xqpi is in interval (0,1] except when xq outside of domain of x.
    """
    nxq, nx = xq.shape[0], x.shape[0]
    
    xi = 0
    x_low = x[0]
    x_high = x[1]
    for xqi_cur in range(nxq):
        xq_cur = xq[xqi_cur]
        while xi < nx-2:
            if x_high >= xq_cur:
                break
            xi += 1
            x_low = x_high
            x_high = x[xi+1]

        xqpi[xqi_cur] = (x_high-xq_cur) / (x_high-x_low)
        xqi[xqi_cur] = xi

# Numba's njit decorator does just-in-time compilation of function,
# short for jit(nopython=True), which throws error if Numba cannot
# successfully compile everything
@njit
def setmin(x, xmin):
    """Set 2-dimensional array x where each row is ascending
    equal to equal to max(x, xmin)."""
    ni, nj = x.shape
    for i in range(ni):
        for j in range(nj):
            if x[i,j] < xmin:
                x[i,j] = xmin
            else:
                break
            

