/*
0_assembledata.do
Hugh Montag
11/29/2023
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)

Goal: Import the French survey data and macroeconomic statistics and clean them 
 
*/

* Directories
global rawpath = "${datapath}\rawdata"
global outpath = "${datapath}\Deriveddata"

// Files to import
* INSEE microdata and processed data
local frdata="${outpath}\CAMMESurveyData"  // CAMME Survey microdata
local frpanel="$outpath"+"\jupyterID"      // CAMME Survey with household id's 

* Inflation Statistics
local inseeoverall="${rawpath}\insee_cpi.csv"  // French CPI (INSEE)
local hicpsubs="${rawpath}\hicp_data.csv"      // HICP price indices
local frsubindM="$outpath\France_cpi_subindices"  // French HICP subindices (e.g. CP011)
local frfoodindM="$outpath\France_cpi_foodindices" // French HICP food-specific subindices
local froop_def ="$rawpath\FROOPList.xlsx"        // Definition of FROOP coicops

* French consumption data
local frHBS = "${rawpath}/FranceHBS2010.xlsx"    // Household Budget Survey for France

* Other Economic Time Series
local macrostats="$rawpath\france_macro_statistics.xls"   // French macroeconomic statistics (eg. unemployment)
local frwheat="$rawpath\insee_wheatprice.csv"             // Wheat commodity prices
local globcommod = "$rawpath\global_commodities.csv"

* Inflation Outputs
local fr1digindM="$outpath\France_cpi_subindices_1dig" // French HICP subindices with 1 digit (e.g. CP05)
local fr4digindM="$outpath\France_cpi_subindices_4dig" // French HICP subindices with 3 or 4 digits (e.g. CP01111)

/* Table of contents
    Step 1: Import Inflation Data
    Step 1a: Import Macroeconomic Data
    Step 1b: Import subcatory prices and merge all data
    Step 2: Run time series statistical tests
    Step 3: Compile CAMME Survey Data
    Step 4: Create French CPI inflation subsets    
*/

pause off

***********************************
* Step 1: Import Inflation Data
***********************************
//Import INSEE year-on-year inflation rates (monthly freq)
clear 
insheet using `inseeoverall', delimiter(";") nonames
drop in 1/3
rename v1 mdatestring
rename v2 inseeinflyoy
rename v3 flags

* Generate a monthly variable
gen mdate=monthly(mdatestring,"YM")
format mdate %tm
destring inseeinflyoy, replace
sort mdate
tsset mdate
gen inseeinflyoyL1=L1.inseeinflyoy
drop flags mdatestring
* Save temporary data
tempfile tempdata
save `tempdata', replace
describe

*------------------------------------
// Import (HICP) French yoy inflation
// Import French yoy inflation 
clear
use `frsubindM'
tsset mdate 
//Drop COICOP level-1 inflation rates (i.e. CP01). We won't be using it here
keep mdate hicpinflyoy hicpinfl3yo3y
gen hicpinflyoyL12=L12.hicpinflyoy
gen hicpinflyoyL24=L24.hicpinflyoy
gen hicpinflyoyL1=L1.hicpinflyoy

//Save a temporary file to be merged in later
merge 1:1 mdate using `tempdata', keep(3) nogenerate
save `tempdata', replace

***********************************
* Step 1a: Import Macroeconomic data
***********************************
//Import French unemployment rage, bond yields, and oil prices
clear
import excel using `macrostats', cellrange(A17:F297) firstrow

//Format date variable, drop unnecessary data
gen mdate=mofd(observation_date)
format mdate %tm
sort mdate
rename LRHUTTTTFRM156S unemprate
rename IRLTLT01FRM156N bondyield

* Generate year-on-year oil inflation
tsset mdate
gen oilinfl=S12.MCOILBRENTEU
keep mdate unemprate bondyield oilinfl

//Merge with other temporary data
merge 1:1 mdate using `tempdata', keep(3) nogenerate
save `tempdata', replace

*-----------------------------
//Load gdp growth (quarterly)
//I will have to save this separately because all other statistics are monthly
clear
import excel using `macrostats', cellrange(H17:J110) firstrow
gen qdate=qofd(observation_date)
format qdate %tq
sort qdate
tsset qdate
rename CLVMNACSCAB1GQFR rgdp_level
gen rgdp_yoy=S4.rgdp_level
tempfile tempdataq
save `tempdataq', replace

*--------------------
//Import French wheat prices
clear
//Import wheat prices
insheet using `frwheat', delimiter(";") nonames
//The first three rows are documentation. Drop them
keep if _n>3
rename v1 date
rename v2 wheat_index
rename v3 varcode

//Covert variables into numeric and date formats
destring wheat_index, replace
gen mdate=monthly(date,"YM")
format mdate %tm

//Generate annual changes in wheat prices
sort mdate
tsset mdate
gen lnprice=ln(wheat_index)
gen wheat_infl=100*S12.lnprice
gen L6wheat_infl=L6.wheat_infl   //Find wheat inflation with 6-mo lag
drop lnprice

//Merge with other temporary macroeconomic data
merge 1:1 mdate using `tempdata', keep(3) nogenerate
save `tempdata', replace

*--------------------
//Import global commodities price index. Series PALLFNFINDEXM in FRED, 
//pulled from https://www.imf.org/en/Research/commodity-prices
clear
import delimited using `globcommod', varnames(1)
gen mdate = mofd(date(date,"YMD"))
format mdate %tm
rename pallfnfindexm commodities_index

sort mdate
tsset mdate
gen lncommodities = ln(commodities_index)
gen commodities_infl = 100*S12.lncommodities

drop date lncommodities

merge 1:1 mdate using `tempdata', nogenerate
sort mdate
* Generate a price index of wheat relative to primary commodities
gen wheatrelative_index = wheat_index / commodities_index
gen lnwheat_relative = ln(wheatrelative_index)
gen wheatrelative_infl = S12.lnwheat_relative
drop lnwheat_relative
gen L6wheatrelative_infl = L6.wheatrelative_infl
save `tempdata', replace

***********************************
* Step 1b: Import subcatory prices and merge all data
***********************************
*--------------------
//Import French bread prices
clear
use `frfoodindM'
sort mdate
//Keep bread, food, meat, liquor, tobacco, gasoline
keep mdate subindyoyCP01113 subindyoyCP011 subindyoyCP0112 ///
subindyoyCP021 subindyoyCP022 subindyoyCP07222
* Generate lagged bread inflation rate
tsset mdate
gen subindyoyCP01113L1=L1.subindyoyCP01113

* Generate food-ex-bread inflation rates
* Estimate of share of bread in food comes from https://statbel.fgov.be/en/themes/households/household-budget-survey-hbs#panel-12
* CP011 - CP0113 in Belgium 2012, SHEET HBS_Tab01_BE_2012
local bread_shareof_food = 289 / 4206
gen subindyoyCP011_ex_CP01113 = (subindyoyCP011 - subindyoyCP01113 * `bread_shareof_food') * 1 / (1 - `bread_shareof_food')
label var subindyoyCP011_ex_CP01113 "Inflation Rate for Food ex. Bread"

merge 1:1 mdate using `tempdata', keep(3) nogenerate
save `tempdata', replace

* Load quarterly real gdp
gen qdate=qofd(dofm(mdate))
format qdate %tq
sort qdate
merge n:1 qdate using `tempdataq', nogenerate keep(master match)
sort mdate qdate
save ${outpath}\macrostats, replace

***********************************
* Step 1c: Create FROOP and non-FROOP inflation rates
***********************************

* Import list of FROOP and not-FROOP coicops
import excel using `frooplist', sheet("COICOP") firstrow clear
rename COICOP coicop
sort COICOP
drop COICOPnum

tempfile froopdata
save `froopdata'

* Merge with household expenditure share data
import excel using `frHBS', sheet("FranceHBS2010") first clear
keep coicop Value
rename Value expend_share
sort coicop
merge 1:1 coicop using `froopdata'
tab _merge

keep if _merge > 1
list if _merge == 2
drop _merge
tab coicop
save `froopdata', replace
sort FROOP
* Find share of household expenditure on FROOP iems
by FROOP: egen expendFROOP_total = total(expend_share)
egen expend_total = total(expend_share)
gen expendFROOP_share = expendFROOP_total / expend_total
keep FROOP expendFROOP_total expendFROOP_share
duplicates drop
list
pause

* Merge in HICP pricing data by coicop
insheet using `hicpsubs', comma names clear
keep if geo=="France"
gen mdate=monthly(time,"YM") 
format mdate %tm
rename value subind
drop unit geo time flagandfootnotes
describe
* Destring subindex values, dropping missing : values;
destring subind, ignore(":") force replace

* Fix some coicops with trailing 0's
replace coicop = substr(coicop, 1, 5) if inlist(coicop,"CP0820", "CP0830")

merge n:1 coicop using `froopdata'
tab _merge
keep if _merge > 1
list if _merge == 2
drop _merge

// Generate FROOP and not-FROOP inflation rates

* Set data as panel format, so I can use time-series operators;
encode coicop, generate(coicopnum) 
sort coicopnum mdate
xtset coicopnum mdate

* Find year-on-year inflation;
gen subindmiss = missing(subind)
tab coicop subindmiss
tab mdate subindmiss

gen lnsubind=ln(subind) 
drop if lnsubind==.
gen subindyoy=100*S12.lnsubind
drop subind lnsubind
sort mdate FROOP coicop subindmiss

* Generate weighted inflation for FROOP and not-FROOP
local var subindyoy
gen wu = expend_share
replace wu = . if `var' == .

by mdate FROOP (coicop), sort: egen totw = total(wu)
gen adjwght = wu / totw

* Create mean 
by mdate FROOP (coicop), sort: egen inflmean = total(adjwght * `var')
drop wu totw adjwght `var' coicop

* Only keep one observation per FROOP-date
by mdate FROOP, sort: gen nm = (_n == 1)
keep if nm == 1
rename inflmean subindyoy

* Reshape wide
gen dum = "FROOP"
replace dum = "NOTFROOP" if FROOP == 0
keep subindyoy mdate dum
replace subindyoy = . if mdate < monthly("2002m1", "YM")
reshape wide subindyoy, i(mdate) j(dum) string
save `froopdata', replace

merge 1:1 mdate using ${outpath}\macrostats, nogen
save ${outpath}\macrostats, replace


***********************************
* Step 1d: Find overall-ex-bread inflation rate
***********************************
* This is an exploratory analysis
use ${outpath}\macrostats, clear

* Generate food-ex-overall inflation rates
* Estimate of share of bread in basket comes from https://statbel.fgov.be/en/themes/households/household-budget-survey-hbs#panel-12
* CP011 - CP0113 in Belgium 2012, SHEET HBS_Tab01_BE_2012
local bread_shareof_basket = 289 / 35429
gen hicpinflyoy_ex_CP01113 = (hicpinflyoy - subindyoyCP01113 * `bread_shareof_basket') * 1 / (1 - `bread_shareof_basket')
label var hicpinflyoy_ex_CP01113 "Inflation Rate for Food ex. Bread"

corr hicpinflyoy_ex_CP01113 hicpinflyoy
display("Bread share of basket is `bread_shareof_basket'")

***********************************
* Step 2: Run time series statistical tests
***********************************
use ${outpath}\macrostats, clear
* Find wheat-bread correlation and test Granger Causality
pwcorr wheat_infl subindyoyCP01113, sig
pwcorr L6wheat_infl subindyoyCP01113, sig
pwcorr L6wheatrelative_infl subindyoyCP01113, sig
tsset mdate
var wheat_infl subindyoyCP01113, dfk small
vargranger 

* Run tests for stationarity
dfuller subindyoyCP01113
dfuller hicpinflyoy

* Run test for stationarity for changes in inflation, 
* to test whether inflation is I(1)
gen hicpinflyoyD=D.hicpinflyoy
gen subindyoyCP01113D=D.subindyoyCP01113
dfuller hicpinflyoyD

*preserve
sort qdate
collapse (mean) hicpinflyoy subindyoyCP01113 subindyoyFROOP subindyoyNOTFROOP, by(qdate)
tsset qdate
dfuller subindyoyCP01113
dfuller hicpinflyoy

* Run vector-autoregressions to parameterize the inflation process in
* the models
display("VAR of FROOP and NOTFROOP")
var subindyoyFROOP subindyoyNOTFROOP, lags(1)
display("Variance-covariance matrix of the errors, FROOP")
matrix list e(Sigma)

display("VAR of bread and not-bread")
var subindyoyCP01113 hicpinflyoy, lags(1)
display("Variance-covariance matrix of the errors, bread")
matrix list e(Sigma)

***********************************
* Step 3: Compile CAMME Survey Data
***********************************
//Link households responses across waves, create French panel, save as separate file
//Load French survey data
clear
use `frdata'
* Generate a primitive ID from all the ident variables
gen primID=numadd 
replace primID=ident if primID==""
replace primID=ident2 if primID==""
drop if wave==.
sort mdate wave ssech primID 
save ${outpath}\cammedata, replace

* Load and merge French id key;
clear 
use `frpanel'
sort primID wave ssech mdate
replace ssech=ssech+1
* Only keep the variables we need to merge with, plus ones created by 
* the ID program;
keep mdate wave ssech primID newwave houseID wavelink* firstdate ssechnum
merge 1:1 mdate wave ssech primID using ${outpath}\cammedata, keep(2 3) nogenerate

save ${outpath}\cammepanel, replace   //Temporary file


***********************************
* Step 4: Create 1-digit and 4-digit inflation rates
***********************************

* Import CPI subindices;
insheet using `hicpsubs', comma names clear
keep if geo=="France"
gen mdate=monthly(time,"YM") 
format mdate %tm
rename value subind
drop unit geo time flagandfootnotes
describe
* Destring subindex values, dropping missing : values;
destring subind, ignore(":") force replace

* Set data as panel format, so I can use time-series operators;
encode coicop, generate(coicopnum) 
sort coicopnum mdate
xtset coicopnum mdate

* Find year-on-year inflation;
gen lnsubind=ln(subind) 
drop if lnsubind==.
gen subindyoy=100*S12.lnsubind
drop subind lnsubind

* Label some variables;
label var coicop "COICOP Code"
label var coicop_label "Text Description of COICOP Code"
label var coicopnum "Numeric version of COICOP Code"
label var subindyoy "Year-on-year inflation rate for COICOP category"

* Reshape this dataset wide, so that we can run regressions of perceptions
* on these categories;
drop coicop_label coicopnum //Drop unnecessary variables for the regression;
sort coicop

tempfile hicpclean
save `hicpclean'

// CREATE 3- and 4-DIGIT COICOP DATASET;
preserve

*keep if (strlen(coicop) == 6 |& substr(coicop,1,2) == "CP");
keep if (inlist(strlen(coicop), 6, 7) & substr(coicop,1,2) == "CP")
sort coicop
gen obs = missing(subindyoy)
by coicop: egen nummiss = total(obs)
keep if nummiss < 24
drop obs nummiss

reshape wide sub*, i(mdate) j(coicop) string //Reshape wide;

sort mdate
* Save the results;
save `fr4digindM', replace

// CREATE 1-DIGIT COICOP DATASET
restore
keep if (inlist(strlen(coicop), 4) & substr(coicop,1,2) == "CP")
sort coicop
gen obs = missing(subindyoy)
by coicop: egen nummiss = total(obs)
keep if nummiss < 24
drop obs nummiss
drop if coicop == "CP00"

list in -10
reshape wide sub*, i(mdate) j(coicop) string   
save `fr1digindM', replace