/*
1_agggraphs.do
Hugh Montag
11/29/2023
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)

Goal: Make aggregate and general graphs of the French data 
 
*/

* Directories
global derpath = "${datapath}\derived_data"
global rawpath = "${datapath}\raw_data"

* Files to import
local frdata = "${derpath}\cammedata"
local frpanel = "${derpath}\cammepanel"

//Cutoff for outliers
local percept_cutoff=20   //Drop all observations where abs(perceptions)>percept_cutoff

****************************************
* Step 1A: Graph aggregated time series
****************************************
//Import French microdata and collapse
clear
use `frdata'
sort mdate
* Truncate the inflation perceptions 
keep if abs(price_com)<=`percept_cutoff'
* Drop observations with negative weights
drop if iweight<=0

* Generate a "fixed" price expectation, where type-4 responses
* are corrected if the gap is larger than 5%
gen evolprice_fix=evolprice_com
replace evolprice_fix=price_com if evolprice_com==0 & abs(price_com)>5
pwcorr evolprice_com price_com
pwcorr evolprice_com evolprice_fix
gen samecom=(evolprice_com==price_com)
tab samecom if price_com != .

//Make a histogram of responses for two dates, fit a curve
gen mdate2=mdate 
label define mstupid 602 "March 2010" 661 "February 2015"
label values mdate2 mstupid

// APPENDIX FIGURE 4
histogram price_com if mdate==monthly("2010m3","YM") | mdate==monthly("2015m2","YM"),  ///
discrete start(-10) width(2) xlabel(-10(5)20)  ///
by(mdate2,  note(""))     ///  title("Perceptions Histogram")
scheme(s1color) xtitle("Percent") 
graph export "${graphpath}\appendfig4.pdf", replace
graph export "${graphpath}\appendfig4.png", replace
window manage close graph    //Close the open graph window

// Collapse to aggregated statistics (mean, median, std)
collapse (mean) pricemean=price_com evolmean=evolprice_com fixmean=evolprice_fix  ///
(p50) pricemed=price_com evolmed=evolprice_com fixmed=evolprice_fix  ///
(sd) pricesd=price_com evolsd=evolprice_com fixsd=evolprice_fix, by(mdate)

//Merge with inflation statistics
merge 1:1 mdate using "${rawpath}\macrostats", keep(3) nogenerate  //Only keep matching results

//Label variables 
label var hicpinflyoy "Actual Inflation"
label var pricemean "Mean Inflation Perceptions"
label var unemprate "Unemployment Rate"

//FIGURE 1: MEAN PERCEPTIONS ON LAGGED INFLATION
graph twoway (line pricemean mdate, lcolor(black) lpattern(solid)) ||  ///
(line evolmean mdate, lcolor (red) lpattern(long_dash)) ||   ///
(line hicpinflyoy mdate, lcolor(green) lpattern(dash_dot)), ///
legend(label(1 "Perceptions") label (2 "Expectations") ///
label(3 "Actual Inflation Rate")) tlabel(, format(%tmCY))  ///
xtitle("Date") ytitle("Percent") scheme(s1color)

graph export "${graphpath}\fig1.pdf", replace
graph export "${graphpath}\fig1.png", replace
window manage close graph    //Close the open graph window

//APPENDIX FIGURE 3 MEAN-MED: MEAN AND MEDIAN MEDIAN PERCEPTIONS ON LAGGED INFLATION
graph twoway (line pricemed mdate, lcolor(black) lpattern(solid)) ||  ///
(line pricemean mdate, lcolor(blue) lpattern(solid)) ||   ///
(line hicpinflyoy mdate, lcolor(green) lpattern(dash_dot)), ///
legend(label(1 "Median Perceptions") label (2 "Mean Perceptions") ///
label(3 "Lagged Inflation")) tlabel(, format(%tmCY))  ///
xtitle("Date") ytitle("Percent") scheme(s1color)

graph export "${graphpath}\appendfig3.pdf", replace
graph export "${graphpath}\appendfig3.png", replace
window manage close graph    //Close the open graph window

sum pricemed, detail
gen meanmeddiff = pricemean-pricemed
sum meanmeddiff, detail
gen meanerror = pricemed - hicpinflyoy
sum meanerror, detail
pause

//GRAPH OVERALL INFLATION, BREAD INFLATION, AND FOOD INFLATION
corr hicpinflyoy subindyoyCP01113 subindyoyCP011
matrix corr_result1 = e(C)
matrix list corr_result1
local hicp_bread_corr = corr_result1[1,2]
local bread_food_corr = corr_result1[2,3]

// FIGURE 2: GRAPH OVERALL, FOOD, AND BREAD INFLATION
graph twoway (line hicpinflyoy mdate, lcolor(black) lpattern(solid)) ||  ///
(line subindyoyCP01113 mdate, lcolor(blue) lpattern(dash_dot)) ||  ///
(line subindyoyCP011 mdate, lcolor(red) lpattern(shortdash)),  ///
legend(label(1 "Overall Inflation") label (2 "Bread Inflation") label (3 "Food Inflation")) ///
tlabel(, format(%tmCY))  ///
xtitle("Date") ytitle("Percent") scheme(s1color)

graph export "${graphpath}\fig2.pdf", replace
graph export "${graphpath}\fig2.png", replace
window manage close graph    //Close the open graph window