/*
2_regressions.do
Hugh Montag
11/29/2023
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)

Goal: Run basic frequency regressions for section 3 
*/

* Directories
global derpath = "${datapath}\derived_data"
global rawpath = "${datapath}\raw_data"

* Files to import
local frdata = "${derdata}\cammedata"
local frpanel = "${derdata}\cammepanel"

* Macro files to import
local fr1digindM="$derdata\France_cpi_subindices_1dig" 
local fr4digindM="$derdata\France_cpi_subindices_4dig" 
local hicpsubs="${rawpath}\hicp_data.csv"

//Cutoff for outliers
local percept_cutoff=20   //Drop all observations where abs(perceptions)>percept_cutoff

//Step 2
//Run regressions on all 2-digit and 4-digit goods? This takes a while
local all_2and4_digitreg_step = 1

****************************************
* Step 0: FROOP Regression
****************************************
* This isn't in the paper yet, I'm just curiuos what the results are
//Load in microdata
clear
* use `frpanel'
use `frdata'
sort mdate
* Truncate the inflation perceptions 
keep if abs(price_com)<=`percept_cutoff'
* Drop observations with negative weights
drop if iweight<=0

//Merge with inflation statistics
merge n:1 mdate using "${derdata}\macrostats", keep(3) nogenerate  //Only keep matching results
eststo clear

reg price_com hicpinflyoy hicpinfl3yo3y, vce(cluster mdate)
*exit

label var price_com "Inflation Perceptions"
label var subindyoyFROOP "FROOP Infl."
label var subindyoyNOTFROOP "Not-FROOP Infl."
label var subindyoyCP01113 "Bread Infl."
label var hicpinflyoy "Overall Infl."

// FROOP Regressions
eststo: reg price_com subindyoyFROOP, vce(cluster mdate)
eststo: reg price_com subindyoyFROOP hicpinflyoy, vce(cluster mdate)
eststo: reg price_com subindyoyCP01113 subindyoyFROOP, vce(cluster mdate)
eststo: reg price_com subindyoyFROOP subindyoyNOTFROOP, vce(cluster mdate)

esttab using "${graphpath}\AppendixTable2.tex", label ///    
b(3) se(3) width(0.8\textwidth) replace   ///    cells(b(fmt(%8.3f)) se(fmt(%8.3f) par star))
alignment(l) booktabs collabels(none)  ///
nomtitle compress nonotes ///   mtitle("(1)" "(2)" "(3)" "(4)")
stats(r2 N, labels("\$R^2\$" "Observation") fmt(%8.3f %8.0f)) 

eststo clear

****************************************
* Step 1: HH-level bread regressions
****************************************
//Load in microdata, generate demographic variables, and conduct the bread-analysis
clear
* use `frdata'
use `frpanel'
sort mdate
* Truncate the inflation perceptions 
keep if abs(price_com)<=`percept_cutoff'
* Drop observations with negative weights
drop if iweight<=0

//Merge with inflation statistics
merge n:1 mdate using "${derdata}\macrostats", keep(3) nogenerate  //Only keep matching results

//Generate useful demographic variables
gen rich=(a_ue==3 | a_ue==4)      //Income
gen hicprich=rich*hicpinflyoy     //Overall inflation interacted with income
gen hicppoor=(1-rich)*hicpinflyoy
gen agebin=1
gen age2=age^2
replace agebin=2 if age>=40 & age<60
replace agebin=3 if age>=60
//Expectations with type-4 errors "fixed"
gen evolprice_notype4=evolprice_com
replace evolprice_notype=price_com if abs(evolprice_com-price_com)>6
label var price_com "Inflation Perceptions"

eststo clear

//================================================
//REGRESSION SET 1: BREAD INFL
//Regression HICPINFLYOY BASE
eststo: quietly reg price_com hicpinflyoy, vce(cluster mdate)
estadd local Sample "Full"
//Regression HICP+BREAD BASE
eststo: quietly reg price_com hicpinflyoy subindyoyCP01113, vce(cluster mdate)
estadd local Sample "Full"
test _b[hicpinflyoy]=_b[subindyoyCP01113]
estadd scalar Ttest=r(p) 

//Regression HICP+BREAD+FOOD_ex_bread
eststo: quietly reg price_com hicpinflyoy subindyoyCP01113 subindyoyCP011_ex_CP01113, vce(cluster mdate)
estadd local Sample "Full"
test _b[hicpinflyoy]=_b[subindyoyCP01113]
estadd scalar Ttest=r(p) 

//Regression HICP+BREAD RICH ONLY
eststo: quietly reg price_com hicpinflyoy subindyoyCP01113 subindyoyCP011_ex_CP01113 if a_ue==4, vce(cluster mdate)
estadd local Sample "High Income"
test _b[hicpinflyoy]=_b[subindyoyCP01113]
estadd scalar Ttest=r(p) 

reg price_com i.a_ue##c.hicpinflyoy i.a_ue##c.subindyoyCP01113 i.a_ue##c.subindyoyCP011 if a_ue==1 | a_ue==4
test _b[1.a_ue#c.subindyoyCP01113]=_b[4.a_ue#c.subindyoyCP01113] 

//Regression HICP+BREAD+HH FIXED EFFECTS
eststo: reghdfe price_com hicpinflyoy subindyoyCP01113 if houseID != ., vce(cluster mdate) absorb(houseID)
estadd local Sample "Full"
test _b[hicpinflyoy]=_b[subindyoyCP01113]
estadd scalar Ttest=r(p) 
estadd local FixEffects "Household"

//Regression HICP+BREAD BASE+MACRO CONTROLS
eststo: quietly reg price_com hicpinflyoy subindyoyCP01113 rgdp_yoy unemprate, vce(cluster mdate)
estadd local Sample "Full"
test _b[hicpinflyoy]=_b[subindyoyCP01113]
estadd scalar Ttest=r(p) 

//Adjust variable descriptions
label var hicpinflyoy "Overall Inflation"
label var subindyoyCP01113 "Bread Inflation"
label var subindyoyCP011 "Food Inflation"
label var subindyoyCP011_ex_CP01113 "Food ex-Bread Inflation"

//Assemble the five regressions to a .tex table and export
esttab using "${graphpath}\tab2.tex", label ///    
b(3) se(3) width(0.8\textwidth) replace   ///    cells(b(fmt(%8.3f)) se(fmt(%8.3f) par star))
alignment(c) booktabs collabels(none)  ///
nomtitle compress nonotes ///   mtitle("(1)" "(2)" "(3)" "(4)")
indicate("Macroeconomic Controls = rgdp_yoy unemprate") ///
stats(Sample FixEffects Ttest N, ///  
labels("Sample" "Fixed Effects" "T-test: Bread Infl. = \\ Overall Infl." "Observations") ///  
fmt(%s %s %8.3f %8.0f)) substitute(\_ _)

eststo clear

****************************************
* Step 2: Regression on all 2- and 4-digit goods
****************************************
// This step creates table 1 in the paper
if `all_2and4_digitreg_step' == 1 {

// Load survey data
use `frpanel', clear
sort mdate
keep if abs(price_com)<=`percept_cutoff'
* Drop observations with negative weights
drop if iweight<=0
merge n:1 mdate using "${derdata}/macrostats", keep(3) nogenerate  

//Merge in 2-digit inflation rates
keep mdate hicpinflyoy price_com subindyoyCP01113
sort mdate 
merge n:1 mdate using `fr2digindM'
keep if _merge == 3
drop _merge

//Merge in 4-digit inflation rates
merge n:1 mdate using `fr4digindM'
keep if _merge == 3
drop _merge

drop subindyoyCP09119

* Create an empty dataset to have the results appended to
preserve
clear
set obs 1
gen x = 2
list
save "${graphpath}\reg1e_2and4dig_regsave.dta", replace
restore

* Regress E \pi = \alpha + \beta \pi + \gamma \pi_sub for each
* 2- and 4-digit coicop code, i.e. CP0314
foreach subCP of varlist subindyoyCP* {
	display("`subCP'")
	preserve
	quietly: reg price_com hicpinflyoy `subCP', vce(cluster mdate)
	*quietly: reg price_com hicpinflyoy `subCP', beta	
	regsave `subCP' using "${graphpath}\reg1e_2and4dig_regsave", tstat pval append
	display("regdone")
	restore
}

* Open the dataset with the results and look at the most significant
* categories
* Import a coicop file description
insheet using `hicpsubs', comma names clear
gen mdate=monthly(time,"YM")
keep if geo=="France" & mdate == monthly("2010m1", "YM")
keep coicop coicop_label
sort coicop
tempfile coicdesc
save `coicdesc', replace

use "${graphpath}\reg1e_2and4dig_regsave", clear
drop if x == 2
drop stderr N x
gen sig5 = (pval < 0.05)
gen abststat = abs(tstat)
gen coicop = substr(var, 10, 7)

sort coicop
merge 1:1 coicop using `coicdesc'
keep if _merge == 3 | _merge == 1
drop _merge
	

sort r2
keep if coef > 0
list coicop coicop_label coef tstat r2 in -10/-1
// This file is used to create TABLE 1 in the paper. The table has to be manually created for formatting
// reasons.
outsheet coicop coicop_label coef tstat r2 using "${graphpath}\list_coicop_2and4.csv" in -10/-1, comma replace

// Export results as .tex table
label var coicop_label "Goods and Services Category"
label var coef "Coefficient"
label var tstat "T-statistic"
label var r2 "R^2"
format coef %8.4f
format tstat %4.2f
format r2 %8.4f
sort coicop
tempfile reg2dat
save `reg2dat'

import excel using `frHBS', sheet("FranceHBS2010") first clear
keep coicop Value
rename Value expend_share
sort coicop
merge 1:1 coicop using `reg2dat'
tab _merge
label var expend_share "Budget Share"

texsave coicop_label coef tstat r2 epend_share in -10/-1 using "${graphpath}\tab1.tex", varlabels replace

}


