/*
3_wheatlevel.do
Hugh Montag
11/29/2023
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)

Goal: Run wheat regression and level bias
*/

* Directories
global derdata = "${datapath}\Deriveddata"

* Files to import
local frdata = "${derdata}\cammedata"
local frpanel = "${derdata}\cammepanel"

//Cutoff for outliers
local percept_cutoff=20   //Drop all observations where abs(perceptions)>percept_cutoff

//Which version of wheat instrument to use? 
//Options are "wheatrelative" or "wheat". Former is wheat relative to commodities index, latter is just wheat prices
local wheatinstr = "wheatrelative"

// Steps to run. Set values to 1 for steps you wish to run
local stepwheat = 1
local steplevel = 0

********************************************
* Step 1: Wheat/Wave Rigor tests
********************************************
if `stepwheat' == 1 {
//Reimport micro data, taking care to match across waves
clear
use `frpanel'
* Drop observations with negative weights
drop if iweight<=0

//Merge with macroeconomic data
sort mdate
keep if abs(price_com)<=`percept_cutoff'
merge n:1 mdate using "${derdata}\macrostats", keep(3) nogenerate

//Generate useful demographic variables
gen rich=(a_ue==3 | a_ue==4)      //Income
gen hicprich=rich*hicpinflyoy     //Overall inflation interacted with income
gen hicppoor=(1-rich)*hicpinflyoy
gen agebin=1
replace agebin=2 if age>=40 & age<60
replace agebin=3 if age>=60
//Expectations with type-4 errors "fixed"
gen evolprice_notype4=evolprice_com
replace evolprice_notype=price_com if abs(evolprice_com-price_com)>6


//REGRESSION SET 2: RIGOR BREAD REGRESSIONS
//Regression WHEAT IV


eststo clear
eststo: ivregress 2sls price_com hicpinflyoy (subindyoyCP01113=L6`wheatinstr'_infl), vce(cluster mdate)
estadd local RegType "IV"
estat firststage
matrix dd=r(singleresults) 
estadd scalar FirstF=dd[1,4]

*exit
//Regression WHEAT IV with demos
eststo: quietly ivregress 2sls price_com hicpinflyoy (subindyoyCP01113=L6`wheatinstr'_infl) rgdp_yoy unemprate bondyield oilinfl, vce(cluster mdate)
estat firststage
estadd local RegType "IV"
matrix dd=r(singleresults) 
estadd scalar FirstF=dd[1,4]
display(dd[1,4])

*-----------------------------------
//Make some changes and delete obs for the differenced regs
//Only keep observations where I have the first and third observations
keep if wavelink1==1 & wavelink3==1
keep if newwave!=2
sort mdate

//Find the size of the update in inflation expectations and perceptions
//Note that I only have wave 1 and wave 3 observations for households
sort houseID mdate
by houseID: gen price_com_update=price_com-price_com[_n-1]
by houseID: gen evolprice_com_update=evolprice_com-evolprice_com[_n-1]
by houseID: gen hicpinflyoy_update=hicpinflyoy-hicpinflyoy[_n-1]
by houseID: gen subindyoyCP01113_update=subindyoyCP01113-subindyoyCP01113[_n-1]
by houseID: gen unemprate_update=unemprate-unemprate[_n-1]
by houseID: gen bondyield_update=bondyield-bondyield[_n-1]
by houseID: gen L6`wheatinstr'_infl_update=L6`wheatinstr'_infl-L6wheat_infl[_n-1]
by houseID: gen oilinfl_update=oilinfl-oilinfl[_n-1]

keep if newwave==3
keep if mdate<=monthly("2014m12","YM")

*ivregress 2sls price_com hicpinflyoy_update (subindyoyCP01113_update=L6wheat_infl_update)

//Regression DIFFERENCED REG
eststo: quietly reg price_com_update hicpinflyoy_update subindyoyCP01113_update, vce(cluster mdate)
estadd local RegType "OLS"

//Regression DIFFERENCED REG with demos
eststo: quietly reg price_com_update hicpinflyoy_update subindyoyCP01113_update ///
rgdp_yoy unemprate_update bondyield_update oilinfl_update, vce(cluster mdate)
estadd local RegType "OLS"

//Adjust variable descriptions
label var hicpinflyoy "Overall Infl."
label var subindyoyCP01113 "Bread Infl."
label var hicpinflyoy_update "Chg. in Infl."
label var subindyoyCP01113_update "Chg. in Bread Inf."

//Assemble the five regressions to a .tex table and export
esttab using "${graphpath}\tab3.tex", label compress  ///    
b(3) se(3) width(\hsize) replace   ///    cells(b(fmt(%8.3f)) se(fmt(%8.3f) par star))
alignment(l) booktabs collabels(none) nonotes ///
mtitle("\$E_t \pi_t\$" "\$E_t \pi_t\$" "\$E_t \pi_t\$" "\$\Delta E_t \pi_t\$" "\$\Delta E_t \pi_t\$") ///  
indicate("Macroeconomic Controls = rgdp_yoy unemprate bondyield oilinfl unemprate_update bondyield_update oilinfl_update") ///
stats(RegType FirstF N, labels("Regression Type" "First Stage \\ F-statistic" "Observation")    ///
fmt(%s %7.2f %8.0f)) ///
substitute(\_ _)

* Clear eststo
eststo clear
}

********************************************
* Step 2: Level Bias
********************************************
if `steplevel' == 1 {
//Reload the full microdataset
clear
use `frdata'
sort mdate
* Truncate the inflation perceptions 
keep if abs(price_com)<=`percept_cutoff'
* Drop observations with negative weights
drop if iweight<=0

//Merge with inflation statistics
merge n:1 mdate using "${derdata}\macrostats", keep(3) nogenerate  //Only keep matching results

//Generate useful demographic variables
gen rich=(a_ue==3 | a_ue==4)      //Income
gen hicprich=rich*hicpinflyoy     //Overall inflation interacted with income
gen hicppoor=(1-rich)*hicpinflyoy
gen agebin=1
replace agebin=2 if age>=40 & age<60
replace agebin=3 if age>=60
gen age2=age^2
//Expectations with type-4 errors "fixed"
gen evolprice_notype4=evolprice_com
replace evolprice_notype=price_com if abs(evolprice_com-price_com)>6
//Perception error
gen percepterror=price_com-hicpinflyoy

*local hardrich if a_ue != 2 & a_ue != 3
keep if a_ue==4 | a_ue==1
gen poor=(a_ue==1)

label define rich1 0 "Low-Income" 1 "High-Income"
label values rich rich1
label var hicpinflyoy "Inflation"
label var subindyoyCP01113 "Bread Inflation"
label var poor "Low-Income"
label var rich "High-Income"
label var hicppoor "Low-Income X Inflation"
label var hicprich "High-Income X Inflation"


//REG 3: INCOME LEVEL BIAS TABLE
eststo: quietly reg price_com hicpinflyoy
eststo: quietly reg price_com poor hicpinflyoy hicppoor 
test poor=0
estadd scalar Ttest=r(p) 
estadd local Test2 "0.001>"
eststo: quietly reg price_com poor hicpinflyoy hicppoor ib4.educ i.sex age age2
test poor=0
estadd scalar Ttest=r(p) 
estadd local Test2 "0.001>"

//Assemble the five regressions to a .tex table and export
esttab using "${graphpath}\tab4.tex", label compress  nonotes ///    
b(3) se(3) width(\hsize) replace   ///    cells(b(fmt(%8.3f)) se(fmt(%8.3f) par star))
alignment(l) booktabs collabels(none)  ///
nomtitles nonotes substitute(\_ _) ///  
indicate("Demographic Controls = *.sex *.educ age age2")    ///
stats(Test2 N, labels("T-test: \\   Low-Income Dummy = 0" "Observation") fmt(%s %8.0f))
eststo clear

}
