/* 
CombineSurveyYears.do
Hugh Montag
11/29/2023
From "On the Welfare Costs of Perception Biases", JCMB (forthcoming)

Goal: Aggregate the French survey data across months and years into one dataset that 
    can be analyzed

The French data archives "Reseau-Quetelet" (https://commande.progedo.fr/en/utilisateur/connexion) 
provides the CAMME microdata as a series of folders
that each correspond to a year. Within each year folder, the data is stored as STATA datasets by month.
This program opens each folder, combines the monthly datasets, and then combines the datasets across
years to form one overall datset.

The year folders are named lil-yyyy. The sufix yyyy does not correspond to a year, but rather the number
of the yearly dataset.

*/

clear
set more off
pause on
#delimit ;

* Filepaths for the survey microdata;
global frpath "\\filer6\Oplcsrv2\Transmit\Montag_H\JMP\ReplicationPackage\raw_data";
local pa0764 = "$frpath"+"\\lil-0764\\Stata\\";
local pa0765 = "$frpath"+"\\lil-0765\\Stata\\";

* Designate which steps to run. Set the locals for steps you wish to run to equal 1.;
local step1 = 1;  // Combining data;
local step2 = 1;  // Destrining and translating variables;
local step3 = 1;  // Cleaning quantative beliefs;


/*================
 Step 1: Combine many years at once
  ================*/;
if `step1'==1 {;
local surveynum 0758 0759 0760 0761 0762 0763 0764 0765 0885 1149;  
local surveylast 1176 1253;

! rm ${frpath}\filelist.txt; 
cap file close myfile; 

* Make a list of filenames;
cd "${frpath}\lil-0757\Stata";
! dir *.dta /a-d /b ?>${frpath}\filelist.txt; 
* Open each file in the lil-0757 folder and append into one folder;
file open myfile using "${frpath}\filelist.txt", read;
file read myfile line;
use `line';
*display("`line'");
save totAgg, replace; 

file read myfile line;
while r(eof)==0 {;
	append using `line';
	file read myfile line;
}; 

file close myfile; cap erase totAgg.dta;
cd $frpath;
save, replace;

* Open each year folder containing data, create a list of STATA files in it, and append the monthly files
* together;
foreach num of local surveynum {;
	cd "$frpath\lil-`num'\Stata";
	! dir *.dta /a-d /b ?>$frpath\filelist.txt; 
	file open myfile using "$frpath\filelist.txt", read;
	file read myfile line;
	while r(eof)==0 {;
		append using `line';
		file read myfile line;
	}; 
	file close myfile; cap erase totAgg.dta;
	cd $frpath; 
	save, replace;
};  
;

// Some CAMME surveys have additional supplements that are stored under different filenames with different
// suffixes. These need to be imported and merged separately;

* lil-0953 has the additional 1409pf supplement. Open this folder and import the supplement;
cd "$frpath\lil-0953\Stata";
! dir *.dta /a-d /b ?>$frpath\filelist.txt; 
file open myfile using "$frpath\filelist.txt", read;
file read myfile line;
while r(eof)==0 {;
	if substr("`line'",-6,2)!="pf"{;
		append using `line';
	};
	file read myfile line; 
};
cd $frpath;
save, replace;
file close myfile; 

* lil-1176 has surveys listed separately;
cd "$frpath\lil-1176\Stata";
! dir *.dta /a-d /b ?>$frpath\filelist.txt; 
file open myfile using "$frpath\filelist.txt", read;
file read myfile line;
while r(eof)==0 {;
	if substr("`line'",-8,2)=="16"{;
		append using `line';
		di "2016 uploaded";
	};
	file read myfile line;
};
cd $frpath; 
save, replace;
file close myfile;

* lil-1253 has surveys listed separately; 
cd "$frpath\lil-1253\Stata";
! dir *.dta /a-d /b ?>$frpath\filelist.txt; 
file open myfile using "$frpath\filelist.txt", read;
file read myfile line;
while r(eof)==0 {;
if substr("`line'",-8,2)=="17"{;
append using `line', force;
di "2017 uploaded";
};
file read myfile line;
};
cd $frpath; 
save, replace;
file close myfile;
erase "$frpath\filelist.txt";
};
else {;
use totAgg;
};
/*================
  Step 2: Translate the variables into English and destring
  ================*/;
* This step renames all of the variables with English names. It 
* also destrings the qualitative responses and assigns them 
* English value labels;  
  
if `step2'==1 {;  
* Variables about Basic info;
destring anenq moisenq ssec vaguee rg vertical tu99, replace;
rename anenq year; label var year "Survey year";
rename moisenq month; label var month "Survey month";
label define month_1 1 "January" 2 "February" 3 "March" 4 "April"
5 "May" 6 "June" 7 "July" 8 "August" 9 "September" 10 "October" 
11 "November" 12 "December"; label values month month_1; 
label var ssech "Month corresponding to the first query";
label values ssec month_1;
rename vaguee wave; label var wave "Wave number";
rename numfa numadd; label var numadd "Variable from the sample: number address file";
label var rg "Region of residence";
label define rg_l
			11 "�le-de-France"
			21 "Champagne-Ardenne"
			22 "Picardie"
			23 "Haute-Normandie"
			24 "Centre"
			25 "Basse-Normandie"
			26 "Bourgogne"
			31 "Nord-Pas-de-Calais"
			41 "Lorraine"
			42 "Alsace"
			43 "Franche-Comt�"
			52 "Pays de la Loire"
			53 "Bretagne"
			54 "Poitou-Charentes"
			72 "Aquitaine"
			73 "Midi-Pyr�n�es"
			74 "Limousin"
			82 "Rh�ne-Alpes"
			83 "Auvergne"
			91 "Languedoc-Roussillon"
			93 "Provence-Alpes-C�te d'Azur"
			94 "Corse";
label values rg rg_1;
rename le lyear; label var lyear "Last digit of the survey year (2 for 2012)";
label var vertical "Sample variable, used for calibration: type of habitat";
label var tu99 "Sample partly urban unit size 1999 (callibration variable)";

* Variables about Expectations;
destring situaeco ecofut chomeurs prix evolprix achats epargner nivpass nivfut, replace;
rename situaeco ecosit; 
label var ecosit "In your opinion, in the last 12 months the general economic situation in France has been";
label var ecofut "In your opinion, over the next 12 months the general economic situation in France will";
label define ecosit_1
    1 "improved significantly"
    2 "improved somewhat"
    3 "stagnated"
    4 "slightly degraded"
    5 "deteriorated sharply"
    9 "do not know";
label define ecofut_1 
    1 "improve significantly"
    2 "improve somewhat"
    3 "stagnate"
    4 "slightly degrade"
    5 "deteriorate sharply"
    9 "do not know";
label values ecosit ecosit_1; label values ecofut ecofut_1; 
rename chomeurs unemployed; 
label var unemployed "Do you think that in the next twelve months, the number of unemployed will";
label define unemployed_1
    1 "greatly increase"
	2 "slightly increase"
	3 "remain stationary"
	4 "slightly lower"
	5 "greatly reduce"
	9 "do not know";
label values unemployed unemployed_1;
rename prix price; label var price "Do you find that, over the last twelve months, prices";
label define price_1
    1 "sharply increased"
	2 "moderately increased"
	3 "slightly increased"
	4 "stagnated"
	5 "decreased"
	9 "do not know";
label values price price_1;
rename prixplus priceup; label var priceup "Value of price increase over last twelve months";
rename prixbais pricedown; label var pricedown "Value of price decrease over last twelve months";
rename evolprix evolprice; 
label define evolprice_1
    1 "it will be faster"
	2 "it will continue unabated"
	3 "it will be slower"
	4 "prices will remain stationary"
	5 "prices will decrease"
	9 "do not know";
label var evolprice "Compared to the last twelve months, what will be the evolution of prices over the next twelve months?";
label values evolprice evolprice_1;
rename evpriplu evolpriceup; label var evolpriceup "Value of price increase over next twelve months"; 
rename evpribai evolpricedown; label var evolpricedown "Value of price decrease over next twelve months"; 
rename achats shopping; 
label define shopping_1
    1 "yes, favorable time"
	2 "neither favorable nor unfavorable"
	3 "no, unfavorable time"
	9 "do not know";
label values shopping shopping_1;
rename epargner ssave; label var ssave "In the current economic situation, do you think that this is the right time to save?";
label define ssave_1 
    1 "yes, certaintly"
	2 "yes, maybe"
	3 "no, probably not"
	4 "no, definitely not"
	9 "do not know";
label values ssave ssave_1;
rename nivpass livsit; label var livsit "In your opinion, over the last twelve months, the standard of living in France has";
rename nivfut livfut; label var livfut "In your opinion, over the next twelve months, the standard of living in France will";
label values livsit ecosit_1; label values livfut ecofut_1;
	
* Variables about Financial stuff;
destring finances finanpas finanfut economis equippas equipfut depenses logement depenslo auto, replace;
rename finances finansit;
label var finansit "Which of the following best describe the present situation of your financial households?"; 
label define finansit_1
    1 "you get to put a lot of money aside"
	2 "you get to put some money aside"
	3 "you just balance your budget"
	4 "you need to dip into your savings"
	5 "you are going into debt"
	9 "do not know";
label values finansit finansit_1;
label var finanpas "Over the past year, the financial situation of your household has"; 
label values finanpas ecosit_1;
label var finanfut "Do you think that, over the next twelve months, the financial position of your household will";
label values finanfut ecofut_1;
rename economis plansave; label var plansave "Do you plan to put aside money in the next 12 months?";
label values plansave ssave_1;
rename equippas purchpas; label var purchpas "Did you make major purchases in the last twelve months?"; 
label values purchpas ssave_1;
rename equipfut purchfut; label var purchfut "Do you intend to make major purchases in the next twelves months?";
label values purchfut ssave_1;
rename depenses expenditure; 
label var expenditure "Over the next twelve months, compared to the past twelve months, do you intend to make major purchases";
rename logement housing; label var housing "Are you planning to buy a new or old hosue over the next twelve months?";
label values housing ssave_1;
rename depenslo renovation; label var renovation "Are you planning to improve or renovate housing over the next twelve months";
label values renovation ssave_1;
label var auto "Do you intend to buy a car within the next twelve months?";
label values auto ssave_1;

* Variables about Demographics;
replace classpro="" if classpro=="*"; replace clasprcj="" if clasprcj=="*";
destring nivetude occupa natempl dejatra statut classpro conjoint etudconj occupacj natempcj
dejtracj statutcj clasprcj, replace;
rename annaiss birthyear; label var birthyear "Year of birth";
rename nivetude educ; label var educ "Level of education";
label define educ_1
    1 "primary or no education"
	2 "secondary"
	3 "technical, commercial"
	4 "superior (tertiary)"
	9 "do not know";
label values educ educ_1;
rename nivetud educ2; label var educ2 "Level of education version 2";
destring educ2, replace;
label values educ2 educ_1;
replace educ=educ2 if educ==.;

rename occupa employed; label var employed "Employment status";
label define employed_1
    1 "yes"
	2 "no, unemployed"
	3 "no, retired"
	4 "no, inactive";
label values employed employed_1;
rename natempl job; label var job "Type of job";
label define job_1 
    1 "Apprentice contract"
	2 "Trainee Business"
	3 "Temp worker (placed by a temp agency"
	4 "Under fixed-term contract (seasonal contract, freelancers, contractors...)"
	5 "Under permanent full-time contract"
	6 "Under permanent part-time contract"
	7 "Intern training, student, (exclusive training)"
	8 "Other (independent, employer, manager, partner)";
label values job job_1;
rename dejatra everwork; label var everwork "Is respondent ever employed?";
label define yesno
    1 "yes"
	2 "no";
label values everwork yesno;
rename statut emptype; label var emptype "Employment type";
label define emptype_1
    1 "State employee"
	2 "Local government employee (incl. public housing/hospitals)"
	3 "Public or national busines employee (including social security)"
	4 "Private sector employee"
	5 "Employee of your own company"
	6 "Independent, employer, manager, partner, co-operator, etc.";
label values emptype emptype_1;
rename classpro profclass; label var profclass "Professional classification";
label define profclass_1
    0 "Other cases"
	1 "Maneuver or skilled worker (OS1, OS2, OS3, etc.)"
	2 "Skilled worker or highly qualified (P1, P2, P3, Ta, OQ, etc.)"
	3 "Executive supervisor of the workers"
	4 "Executive foreman technicians or other supervisors"
	5 "Technician, draftsman, VRP"
	6 "Teacher, social worker, nurse"
	7 "Engineer or manager"
	8 "Professor and Class A Personal Public Service"
	9 "Clerk, shop assistant, service agent, caregiver, babysitter";
label values profclass profclass_1;
rename conjoint spouse; label var spouse "Existence of a spouse";
rename annaiscj spbirthyear; label var spbirthyear "Year of birth of spouse";
rename etudconj speduc; label var speduc "Level of education of spouse";
label values speduc educ_1;
rename occupacj spemployed; label var spemployed "Spouse employment status";
label values spemployed employed_1;
rename natempcj spjob; label var spjob "Type of job of spouse";
label values spjob job_1;
rename dejtracj speverwork; label var speverwork "Is the spouse ever employed?";
label values speverwork yesno;
rename statutcj spemptype; label var spemptype "Employment type of spouse";
label values spemptype emptype_1;
rename clasprcj spprofclass; label var spprofclass "Professional classification of spouse";
label values spprofclass profclass_1;

* Variables about Family;
destring quirepon sexe pf, replace;
rename nbenfant numkids; label var numkids "Number of children under 14 in the household";
rename nbactif numadults; label var numadults "Number of adults (?) in the household";
rename nbpers pop; label var pop "Population of household";
rename quirepon respondent; label var respondent "Person respondent";
label define respondent_1
    1 "Family head"
	2 "Spouse of family head";
label values respondent respondent_1;
rename sexe sex; label var sex "Sex of the respondent";
label define sex_1 
    1 "Male"
	2 "Female";
label values sex sex_1;
rename dage age; label var age "Age of respondent at time of 1st survey wave"; 
rename dagecj spage; label var spage "Age of spouse";
rename pf plat; label var plat "Presence of platform";
label define plat_1 
    0 "no"
	1 "yes";
label values plat plat_1;

label var iweight "Weighting coefficient";
destring code_cs_cj_r code_cs_t_r, replace;
label var code_cs_cj_r "Socio-professional category of the joint";
label var code_cs_t_r "Occupational category of consolidated holder";
label define code_cs_1
    00 "other cases"
	10 "farmers"
	21 "crafts"
	22 "shopkeepers or similar"
	23 "CEOs of 10 or employees"
	31 "liberal profession"
	32 "public service executive, intellectual profession, artist"
	36 "corporate executives"
	41 "associate professional in education, health, public or allied"
	46 "general and administrative associate professionals"
	47 "technicians"
	48 "foremen, supervisers"
	51 "public service employee"
	54 "corporate administrative employee"
	55 "commercial employees"
	56 "personal services worker"
	61 "skilled"
	66 "unskilled"
	69 "agricultural workers";
label values code_cs_cj_r code_cs_1; label values code_cs_t_r code_cs_1;

* Variables about Harominzed consumer questions;
destring a_ue c_ue d_ue e_ue pmen_ue, replace;
label var a_ue "European; breakdown of households by income quartile";
label define a_ue_1
    1 "income below the 1st quartile"
	2 "income between the 1st and 2nd quartile"
	3 "income between the 2nd and 3rd quartile"
	4 "income above the 3rd quartile";
label values a_ue a_ue_1;
replace b_ue="" if b_ue=="N" | b_ue=="X"; destring b_ue, replace;
label var b_ue "European; occupation";
label define b_ue_1
    0 "unemployed"
	1 "self-employed professional"
	2 "self-empmloyed farmers"
	3 "clerical employees"
	4 "skilled manual workers"
	5 "other manual workers"
	6 "not used"
	7 "other occupations"
	9 "don't know";
label values b_ue b_ue_1;
label var c_ue "European; working status";
label define c_ue_1 
    1 "Full-time"
	2 "Part-time"
	0 "Don't know again"
	9 "Don't know";
label values c_ue c_ue_1;
label var d_ue "European; education";
label define d_ue_1
    1 "Primary"
	2 "Secondary"
	3 "More than secondary";
label values d_ue d_ue_1;
label var e_ue "European; age group";
label define e_ue_1
    1 "16-29"
	2 "30-49"
	3 "50-64"
	4 "65+";
label values e_ue e_ue_1;
replace f_ue="" if f_ue=="n"; destring f_ue, replace;
label var f_ue "European; gender";
label values f_ue sex_1;
label var pmen_ue "European; Number of persons in HH";
label define pmen_ue_1 
    1 "1"
	2 "2"
	3 "3"
	4 "4+";
label values pmen_ue pmen_ue_1;
replace cscal_ue="" if cscal_ue=="X"; destring cscal_ue, replace;
label var cscal_ue "European: Occupation";
label define cscal_ue_1 
    1 "artisans, professions"
	2 "farmers"
	3 "managers, teacher interm, employees"
	4 "skilled workers"
	5 "unskilled"
	6 "inactive retirees"
	7 "unemployed or other inactive";
label values cscal_ue cscal_ue_1;

*
* Variables about Housing supplement;
rename typlog homtype; label var homtype "Is your home a....";
rename stoc homstat; label var homstat "Are you an owner or a tenant?";
rename propri homown; label var homown "Who owns the home?";
rename alloc homsubsidy; label var homsubsidy "Do you get a housing subsidy?";
rename date_em hom_date; label var hom_date "When did you move into this home?";
rename annees hom_year; label var hom_year "Did you move into this home >4 years?";
rename nplog homrooms; label var homrooms "How many roomsa re in this home?";
rename dif_remb diff_mort; 
label var diff_mort "Over the past 12 months, have you had difficulty paying mortgages?";
rename dif_loye diff_rent; 
label var diff_rent "Over the past 12 months, have you had difficulty paying rent?";
rename impaye outstand; label var outstand "In the last 12 months, has rent/mortgage been outstanding?";
rename bais_rev incomesit; label var incomesit "In the last 12 months, has your income gone down?";
rename aug_log exp_up; label var exp_up "In the past 12 months, have you seen an increase in housing expenses?";
rename aug_cha charge_up; label var charge_up "This cost increase is due to charges?";
rename aug_loyemp rent_up; label var rent_up "This cost increase is due to rent or mortgages?";
rename autres other_up; label var other_up "This cost increase is due to misc?";
rename froid hom_cold; label var hom_cold "During the last winter, was it cold in your dwelling?";
rename travaux hom_stop; label var hom_stop "During the last 12 months, did you have to stop renovating your home?";
rename pxevolp homprfut; label var homprfut "During the next 12 months, will used home prices near you go up?";
rename pxevolf homprfut2; label var homprfut2 "During the next 12 months, will used home prices near you go up?";

save, replace;
};
;

/*==========
  Step 3: Convert expected inflation questions into one variable;
  ==========*/;
// The CAMME asks multiple questions about perceptions and expectations. These need to be combined
// to obtain one quantitative value;
if `step3'==1 {;
clear; use totAgg;
* Generate monthly date variable;
gen mdate=ym(year,month); format mdate %tm;

* Generate variables for quantitative inflation stuff;  
gen price_com=.; gen evolprice_com=.;
* Unspool numbers plus qualitative response;
replace price_com=priceup if inlist(price,1,2,3);
replace price_com=-1*pricedown if price==5;
replace price_com=0 if price==4;

replace evolprice_com=evolpriceup if inlist(evolprice,1,2,3);
replace evolprice_com=-1*evolpricedown if evolprice==5;
replace evolprice_com=0 if evolprice==4;

* Generate dummy variable for missing beliefs;
gen price_missing=missing(price_com); 
gen evolprice_missing=missing(evolprice_com);

* For post-2016 surveys, generate age from birthdate variable;
replace age=year-birthyear if age==. & year>=2016;

* Output microdata dataset;
save CAMMESurveyData, replace;
};
;

delimit cr;